#include "windows.h"
#include "windowsx.h"
#include "camera.h"
#include <GL/GLee.h>
#include <gl/glut.h>



void Camera::setFreeNavCam(Vec3f center, bool bWindowHasFocus, int delta_x, int delta_y)
{
	const float M_PI = 3.14159265358979f;

	static int lbut_state = 0;
	static int mbut_state = 0;
	static int rbut_state = 0;
	
	static float radius = 100.0f;
	static float y_angle = 0;
	static float theta_angle = (M_PI/2.0)-0.05;

	int w = glutGet((GLenum)GLUT_WINDOW_WIDTH);
	int h = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
	
	if(bWindowHasFocus)
	{
		lbut_state = ( GetAsyncKeyState(VK_LBUTTON) & 0x8000 ) ? 1 : 0;
		mbut_state = ( GetAsyncKeyState(VK_MBUTTON) & 0x8000 ) ? 1 : 0;
		
		
		// UPDATE radius, y_angle, AND theta_angle FROM USER INPUT
		if(mbut_state)
		{
			radius -= delta_y * 0.3;
			radius = max(0.1, radius);
		}
		// ROTATE AROUND ORIGIN
		if(!mbut_state) //lbut_state)
		{
			y_angle		+= delta_x * 0.3 * M_PI/180.0;
			theta_angle += delta_y * 0.3 * M_PI/180.0;
			
			theta_angle  = min((M_PI/2.0)-0.05, theta_angle);
			theta_angle  = max(-(M_PI/2.0)+0.05, theta_angle);
			if(y_angle >  M_PI*2) y_angle -= M_PI*2;
			if(y_angle < -M_PI*2) y_angle += M_PI*2;
		}
	}
	

	// SET OPENGL CAMERA
	Vec3f viewat(0,0,0); 
	Vec3f viewpos(0,0,1); 	
	const Vec3f viewup(0,1,0); 
	const Vec3f yaxis(0,1,0);
	Vec3f tmpvec;
	Mtx3f rotmat;
	
	// Rotate around y-axis
	viewpos = Vec3f(0,0,radius);
	rotmat.rotAxis(yaxis, y_angle);				// create the rotation matrix 
	tmpvec = rotmat*viewpos;			 
	viewpos = tmpvec;							// update viewvec
	
	// Rotate around y-plane
	tmpvec = viewpos%yaxis;
	tmpvec.normalize();							// axis in y-plane to rotate around
	rotmat.rotAxis(tmpvec, theta_angle);		// create the rotation matrix 
	tmpvec = rotmat*viewpos;			 
	viewpos = tmpvec;							// update viewvec
	
	
	/* set projection matrix */
	glMatrixMode(GL_PROJECTION);  
	glLoadIdentity();           /* reset */
	gluPerspective(45.0,(float)w/(float)h, 0.1,10000.0); /* FOV, aspect ratio, near, far */
	
	/* set modelview matrix */
	glMatrixMode(GL_MODELVIEW);  
	glLoadIdentity(); 
	
	gluLookAt(viewpos[X]+center[0],viewpos[Y]+center[1],viewpos[Z]+center[2],
		viewat[X]+center[0],viewat[Y]+center[1],viewat[Z]+center[2],
		viewup[X],viewup[Y],viewup[Z]);

	m_lookFrom = Vec3f(viewpos[X]+center[0],viewpos[Y]+center[1],viewpos[Z]+center[2]);
	m_lookAt   = Vec3f(viewat[X]+center[0],viewat[Y]+center[1],viewat[Z]+center[2]);
	m_upVec	   = Vec3f(viewup[X],viewup[Y],viewup[Z]);
	
}

void Camera::setPlayerView(Vec3f lookFrom, Vec3f lookVec, Vec3f upVec)
{
	int w = glutGet((GLenum)GLUT_WINDOW_WIDTH);
	int h = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
	
	/* set projection matrix */
	glMatrixMode(GL_PROJECTION);  
	glLoadIdentity();           /* reset */
	gluPerspective(45.0,(float)w/(float)h, 0.1,10000.0); /* FOV, aspect ratio, near, far */
	
	/* set modelview matrix */
	Vec3f lookAt = lookFrom + lookVec;
	glMatrixMode(GL_MODELVIEW);  
	glLoadIdentity(); 
	gluLookAt(lookFrom[0], lookFrom[1], lookFrom[2],
		lookAt[0], lookAt[1], lookAt[2],
		upVec[0],  upVec[1],  upVec[2]);
	
	m_lookFrom = lookFrom;
	m_lookAt   = lookAt;
	m_upVec	   = upVec;
	
}

