#ifndef FRACTAL_H
#define FRACTAL_H

//#include "linmath.h"
#include "Noisetab.h"
#include "Noise.h"

#ifndef FALSE
#define FALSE               0
#endif
#ifndef TRUE
#define TRUE                1
#endif


#define MAX_OCTAVES 10
#define BP	0x100
#define BM	0xff

#define N	0x100000

static int p[BP + BP + 2];
static double g[BP + BP + 2][3];

	/* cubic spline interpolation */
#define s_curve(t)	( t * t * (3. - 2. * t) )
	/* linear interpolation */
#define lerp(t, a, b)	( a + t * (b - a) )
#define at3(rx,ry,rz)	( rx * q[0] + ry * q[1] + rz * q[2] )
#define at2(rx,ry)	( rx * q[0] + ry * q[1] )
#define setup(u,b0,b1,r0,r1)\
	t = u /*+ N*/;\
	b0 = ((int)t) & BM;\
	b1 = (b0+1) & BM;\
	r0 = t - (int)t;\
	r1 = r0 - 1.;


double
Noise2(Vec3d vec)
{
	int	bx0, bx1, by0, by1, b00, b10, b01, b11;
	double	rx0, rx1, ry0, ry1, *q, sx, sy, a, b, t, u, v, result;
	int i, j;

//	return(g[p[(int)vec[0]&255]][0]);

#ifdef UNLIMITED_NOISE_DOMAIN
	filter_args(&vec.x, &vec.y, &vec.z);
#endif

	setup(vec[X], bx0,bx1, rx0,rx1);
	setup(vec[Y], by0,by1, ry0,ry1);

	i = p[ bx0 ];
	j = p[ bx1 ];

	b00 = p[ i + by0 ];
	b10 = p[ j + by0 ];
	b01 = p[ i + by1 ];
	b11 = p[ j + by1 ];

	sx = s_curve(rx0); //rx0; 
	sy = s_curve(ry0); //ry0; 

	q = g[ b00 ];		/* get random gradient */
	u = at2(rx0,ry0);	/* get weight on lo x side (lo y) */
	q = g[ b10 ];
	v = at2(rx1,ry0);	/* get weight on hi x side (lo y) */
	a = lerp(sx, u, v);	/* get value at distance sx between u & v */

				/* similarly at hi y... */
	q = g[ b01 ];	u = at2(rx0,ry1);
	q = g[ b11 ];	v = at2(rx1,ry1);
	b = lerp(sx, u, v);

	result = 1.5 * lerp(sy, a, b);	/* interpolate in y */

	return (result);
}  /* Noise2() */

void
Init_Noise()
{
	int i;
	
	for (i= 0; i < BP+BP+2; ++i)
	{
		p[i]= p_precomputed[i];
		g[i][0]= g_precomputed[i][0];
		g[i][1]= g_precomputed[i][1];
		g[i][2]= g_precomputed[i][2];
	}
}

// OBS, MUST ACTUALLY USE Noise3 since we do not use a height field !!!!
//

// Repetitive outside [-1,1], To avoid repetition: scale pnt to always 
// have values in [-1,1] (scale will depend on size of world). Also set
// octaves accordingly.

/* 
 * Ridged multifractal - FKM 4/94
 *
 * Some good parameter values to start with:
 *
 *     H:            0.9
 *     offset:       1
 *     gain:         2
 */
float
calc_noise(Vec3d pnt);

// H = fractal increment parameter (roughness)
// lacunarity = gap between successive frequencies
// octaves = number of frequencies (more frequencies also gives more roughness)
// offset = controls how high up on the mountains fractality starts 
// weight makes flater land at low levels and rougher at high levels. How much is controlled by
// threshold.
double 
RidgedMultifractal( Vec3d point, double H, double lacunarity,
		    double octaves, double offset, double threshold )
{
	double  	result, signal, weight;
	int             i;
	static int      first = TRUE;
	static double   exponent_array[MAX_OCTAVES];

	/* precompute and store spectral weights */
	if ( first ) {

		// compute weight for each frequency 
		for (i=0; i<MAX_OCTAVES; i++) 
			exponent_array[i] = pow( lacunarity, -i*H );
		
		first = FALSE;
	}

	/* get first octave */
	// Ordinary fractal noise function between [-1,1]
	signal = calc_noise( point );
	/* get absolute value of signal (this creates the ridges) */
	// We get "upside down" ridges were the fractal noise goes below zero
	if ( signal < 0.0 ) signal = -signal;
	/* invert and translate (note that "offset" should be ~= 1.0) */
	// since the ridges are upside down we flip the result
	signal = offset - signal;
	/* square the signal, to increase "sharpness" of ridges */
	signal *= signal;
	/* assign initial values */
	result = signal;
	weight = 1.0;

	for( i=1; weight>0.001 && i<octaves; i++ ) {
		point[X] *= lacunarity; // This frequency point will move lacunarity times as fast through noise space between adjacent calls to RidgedMultiFractal
		point[Y] *= lacunarity;
		point[Z] *= lacunarity;

		/* weight successive contributions by previous signal */
//		weight = signal * threshold;
		if ( weight > 1.0 ) weight = 1.0;
		if ( weight < 0.0 ) weight = 0.0;
		signal = calc_noise( point );
		if ( signal < 0.0 ) signal = -signal;
		signal = offset - signal;
		signal *= signal;
		/* weight the contribution */
		signal *= weight;
		result += signal * exponent_array[i];
	}

	return( result );

} /* RidgedMultifractal() */


#define SIZE           64  
#define SIZE_1         65  

float   noise[SIZE+1][SIZE+1][SIZE+1];

float
calc_noise(Vec3d pnt)
{
  float  t1;
  float  p_l,p_l2,    /* value lerped down left side of face1 & face 2 */
         p_r,p_r2,    /* value lerped down left side of face1 & face 2 */
         p_face1,     /* value lerped across face 1 (x-y plane ceil of z) */
         p_face2,     /* value lerped across face 2 (x-y plane floor of z) */
         p_final;     /* value lerped through cube (in z)                  */

  extern float noise[SIZE_1][SIZE_1][SIZE_1];
  register int      x, y, z,px,py,pz;
  
  px = (int)pnt[X];
  py = (int)pnt[Y];
  pz = (int)pnt[Z];
  x = px &(SIZE-1); /* make sure the values are in the table           */
  y = py &(SIZE-1); /* Effectively, replicates the table thought space */
  z = pz &(SIZE-1);
  
  t1 = pnt[Y] - py;
  p_l  = noise[x][y][z+1]+t1*(noise[x][y+1][z+1]-noise[x][y][z+1]);  
  p_r  =noise[x+1][y][z+1]+t1*(noise[x+1][y+1][z+1]-noise[x+1][y][z+1]);
  p_l2 = noise[x][y][z]+ t1*( noise[x][y+1][z] - noise[x][y][z]);     
  p_r2 = noise[x+1][y][z]+ t1*(noise[x+1][y+1][z] - noise[x+1][y][z]);
  
  t1 = pnt[X] - px; 
  p_face1 = p_l + t1 * (p_r - p_l);
  p_face2 = p_l2 + t1 * (p_r2 -p_l2);
  
  t1 = pnt[Z] - pz;
  p_final =  p_face2 + t1*(p_face1 -p_face2);
  
  return(p_final);
}


void Init_3DNoise()
{
  long    i,j, k, ii,jj,kk;
 
  for (i=0; i<SIZE; i++)
   for (j=0; j<SIZE; j++)
     for (k=0; k<SIZE; k++)
       {
         noise[i][j][k] = 1-2*(float)((float)rand()/(float)RAND_MAX);
       }

/* This is a hack, but it works. Remember this is only done once.
 */

 for (i=0; i<SIZE+1; i++)
   for (j=0; j<SIZE+1; j++)
     for (k=0; k<SIZE+1; k++)
       {
         ii = (i == SIZE)? 0:  i;
         jj = (j == SIZE)? 0:  j;
         kk = (k == SIZE)? 0:  k;
         noise[i][j][k] = noise[ii][jj][kk];
       }

}

#endif // FRACTAL_H