#pragma warning (disable:4786)

// NOTE that this project uses NLHawk which requires use of 
// Multithreaded DLL in Project Settings ->C/C++ ->Code Generation -> Use runtime Library

// WARNING: If windows.h is included, it must always be the very first include file. Otherwise lots of compile errors appear.
#include "windows.h"	
#include "winuser.h"
#include <GL/GLee.h>
#include <gl/glut.h>

#include "game.h"
#include "tex/texturemanager.h"

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

static GLenum errCode;
#define CHECK_ERROR 	errCode = glGetError();	\
	if (errCode != GL_NO_ERROR) {	\
	printf("(%s, %d) ERROR: %s\n", __FILE__, __LINE__, gluErrorString(errCode)); \
	}

// class missile
// create missile
// (update missiles)
// (collision detection sets status of objects)
// update -- checkInput
//		  -- updateRemotePlayers, i.e., explosion texture
//		  -- updateMissiles, posision and explosion texture
// Missiles are totally defined by their start values, 
// so each aplication animates them by themselves (client-wise) 
// Collision detection is done between player0 and all (enemy) missiles
// Each application (client) sends message when it gets killed and by which missile.


#define M_PI  3.14159265358979f

Game::Game() : m_viewportWidth(512), m_viewportHeight(512), m_navigationMode(LOCKED_NAV)
{
	int port = 1250; //  34863; // just something pretty random and above 1024
	m_socket.connect(port);

	m_players.clear();
	m_missiles.clear();

	m_timer.start(true);

	m_timeLastInputCheck = m_timer.elapsed();
}

Game::~Game()
{
	// deallocate
}

int Game::getUniqueID()
{
	// A unique number is used to identify each object, such as player or missile
	return rand();
}


void Game::InitGame()
{
	// Seed the random generator with something unique for each player
	srand((unsigned int) time(0)); // we use current time to seed the random generator
	m_playerID = getUniqueID();
	
	// Create ourselves
	// i.e., init player 0
	Player* player0 = new Player();		// "new" is not fast, but this does not happen every frame
	player0->initPlayer0();
	m_players.insert(Playermap::value_type(m_playerID, player0)); // associate player0 with the playerID and insert into the map

//	// Create a dummy player 1
//	Player* player1 = new Player();		// "new" is not fast, but this does not happen every frame
//	unsigned int player1ID = getUniqueID();
//	player1->initPlayer0();
//	m_players.insert(Playermap::value_type(player1ID, player1)); // associate player0 with the playerID and insert into the map
	

	// Preload texture animations
	Object::loadExplosionTextures();
	Missile::loadExplosionTextures();

	// Preload sounds
	m_fireSound.loadWav("Sound/explosion2.wav");
	m_explSound.loadWav("Sound/explosion.wav");
	
	m_bgSong.loadMP3("Sound/Theme - Airwolf.mp3");
	m_bgSong.setAttributes(50);
	//m_bgSong.play();

	// --- Create the surroundings ---
	// We want the same fractal mountain for all players, so set the random generator to something deterministic
	srand(1000);

	m_nature.createMountain();
	m_nature.createTrees();

	// Restore the random seed to something individual per player
	srand((unsigned int) time(0)); // we use current time to seed the random generator
	
}

void Game::initRemotePlayer(unsigned int playerID)
{
	// init the other player(s)
	Player* player = new Player;
	player->initRemotePlayer();
	m_players.insert(Playermap::value_type(playerID, player)); // associate player with the playerID and insert into the map
}

void Game::updateFrame()
{

	synchronizeStatus();		// synchronize game status including player positions, new missiles, dead missiles, dead players

	//-------------------------
	// Update game status
	collisionDetection();		// do collision detection between players and missiles

	updatePlayers();			// update animations and status for objects 

	updateMissiles();			// update animations, status and position for missiles and test collision with ground

	checkInput();				// update ourselves - e.g. position and firing
	//-------------------------
	
	sendOurNewStatus();
}

void Game::renderFrame()
{
	glClearColor(0,0,0,1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	getMouse();	// Get delta mouse movement

	updateFrame();
	
	setupLights();
	setCamera();
	//drawFloor();	
	m_nature.renderMountain();
	m_nature.renderTrees(m_Camera);
	
	renderObjects();
	
	printFrameRate();
	printScores();
	drawCursor();
	
	setMouse();	// eventually we have to wrap mouse position so it does not get stuck at screen borders

	glutSwapBuffers();
}

void Game::setCamera()
{
	Playermap::iterator it = m_players.find(m_playerID);	// look up ourselves. 
	Vec3f lookFrom, lookVec, upVec;
	it->second->getPositionAndOrientation(lookFrom, lookVec, upVec);

	if (m_navigationMode == LOCKED_NAV) 
	{
		m_Camera.setPlayerView(lookFrom, lookVec, upVec);
	}
	else if(m_navigationMode == FREE_NAV)
	{
		m_Camera.setFreeNavCam(lookFrom, windowHasFocus(), m_mouseDeltaX, m_mouseDeltaY);
	}
}

void Game::renderObjects()
{
	Playermap::iterator it; 
	for(it = m_players.begin(); it != m_players.end(); it++)
		it->second->render(m_Camera);

	Missilemap::iterator it2; 
	for(it2 = m_missiles.begin(); it2 != m_missiles.end(); it2++)
		it2->second->render(m_Camera);
	
}

void Game::checkInput()
{
	if(! windowHasFocus())
		return;
	
	Playermap::iterator it = m_players.find(m_playerID);	// look up ourselves. We now acces ourselves with it->second
	Player* ourselves = it->second;
	
	if ( (ourselves->m_status == Object::DEAD) || (ourselves->m_status == Object::EXPLODING)) 
		return;

	//---------------------------------------------
	// Move ourselves

	// This updates the camera, i.e., our player position
	Vec3f pos, lookVec, upVec;
	ourselves->getPositionAndOrientation(pos, lookVec, upVec);
	m_navigator.m_upvec = Vec3f(0,1,0);
	m_navigator.m_lookAt = pos+lookVec;
	m_navigator.m_lookFrom = pos;
	double t = m_timer.elapsed();
	double dt = (t-m_timeLastInputCheck);
	if(dt > 1.0) 
		dt = 0;	// if the window has lost focus for a while we do not want the speed to be madly high
	m_timeLastInputCheck = t;
	m_navigator.m_speed = dt*20.0f;	// set moving speed independent of the current frame rate 
	m_navigator.handleInput(m_mouseDeltaX, m_mouseDeltaY);
	lookVec = m_navigator.m_lookAt - m_navigator.m_lookFrom;
	lookVec.normalize();

	// Check collision with the ground. If so, put us just above.
	pos = m_navigator.m_lookFrom;
	float h = m_nature.getHeight(pos[0], pos[2]);
	pos[1] = h + ourselves->getRadius(); //max(pos[1], h + ourselves->getRadius());

	it->second->setPositionAndOrientation(pos, lookVec, m_navigator.m_upvec);
	
	//---------------------------------------------
	
	//---------------------------------------------
	// Check for firing
	if( (GetAsyncKeyState(VK_LBUTTON) & 0x8000 ) || ( GetAsyncKeyState(' ') & 0x8000 ) )
	{
		// if left mouse button or space is pressed

		// Check that enough time has elapsed since last shot
		double timeSinceLastShot = m_timer.elapsed() - m_timeLastShot;
		if ( timeSinceLastShot > 0.5) // unit is seconds
		{
			m_timeLastShot = m_timer.elapsed();
			
			Missile* missile = new Missile;
			missile->m_ownerID = m_playerID;
			
			missile->setBaseTexture(TextureManager::load("media/textures/missile.TIF"));

			Vec3f lookFrom, lookVec, upVec;
			ourselves->getPositionAndOrientation(lookFrom, lookVec, upVec);
			float r = ourselves->getRadius() + missile->getRadius();
			Vec3f displace = lookVec * r;

			Mtx4f m0;
			m0.translate(displace[0], displace[1], displace[2]);
			missile->m_mtx = m0 * ourselves->m_mtx; // translate just in front of player 0
			missile->m_speedVec = lookVec;
			
			int id = getUniqueID();
			m_missiles.insert(Missilemap::value_type(id, missile));

			// send message that a new missile exists
			Mtx4f m = missile->m_mtx;
			char msg[1024];
			sprintf(msg, "newenemymissile %d %d", m_playerID, id);
			int len = strlen(msg);
			writeDataToString(msg+len+1, &m, sizeof(Mtx4f));		// copy matrix into message
			//m_sendSocket.sendMessage(msg, len+1+sizeof(Mtx4f) );
			m_socket.sendMessage(msg, len+1+sizeof(Mtx4f) );

			// Play sound
			m_fireSound.play();
			
		}
	}
		
}

void Game::updatePlayers()
{
	Playermap::iterator it; 
	
	double t = m_timer.elapsed();
	
	for(it = m_players.begin(); it != m_players.end(); it++)
		it->second->update(t);
}

void Game::updateMissiles()
{
	Missilemap::iterator it; 

	double t = m_timer.elapsed();

	for(it = m_missiles.begin(); it != m_missiles.end(); it++)
		it->second->update(t, &m_nature, &m_explSound);
	
	// remove dead missiles. They cannot be removed inside the update method
	// since removing affects the loop-iterator "it".
	for(it = m_missiles.begin(); it != m_missiles.end(); )
	{
		if(it->second->m_status == Object::DEAD)
			it = m_missiles.erase(it);
		else
			it++;
	}

}

void Game::synchronizeStatus()
{
	// Poll packets from opponents
	const char* message=0;

	while( *(message = m_socket.receiveMessage()) != '\0') // loop while there are messages
	{
		// choose a simple and compact message syntax. 
		// E.g:
		// "p"						playerID		matrix					- message for updating matrix of a player
		// "newenemymissile"		playerID		missile_id		matrix	- message for announcing a new missile
		// "playerdeadbymissile"	enemy_id		missile_id				- message for announcing a player has been killed and by which missile
		//

		char head[100];
		int id = -1;
		Mtx4f m;

		if(sscanf(message, "%s", head))
		{
			if (strcmp(head, "p") == 0) // update enemy player
			{
				if(sscanf(message, "%s %d", head, &id) ) // read id-part of message
				{
					if(id != m_playerID)  
					{
						// read matrix from message
						readDataFromString(message + strlen(message)+1, &m, sizeof(Mtx4f));		// copy the matrix-part of message into matrix
						
						// Update player positions
						Playermap::iterator it = m_players.find(id);
						
						if (it == m_players.end())
						{
							initRemotePlayer(id);
							it = m_players.find(id);
						}
						
						it->second->m_mtx = m; // updating the player's transform matrix
					}
				}
			}
			else if ( strcmp(head, "newenemymissile") == 0 ) // new enemy missile exists
			{
				int playerID = -1;
				int missileID = -1;
				if(sscanf(message, "%s %d %d", head, &playerID, &missileID) ) // read id-parts of message
				{
					Missilemap::iterator it = m_missiles.find(missileID);
					if (it == m_missiles.end())
					{
						// this missile is not already inserted! E.g. it is for instance not our own shot missile.

						// read matrix from message
						readDataFromString(message + strlen(message)+1, &m, sizeof(Mtx4f));		// copy the matrix-part of message into matrix
						
						Missile* missile = new Missile;
						missile->m_ownerID = playerID;
						
						missile->setBaseTexture(TextureManager::load("media/textures/missile.TIF")); // NOTE: filename is case sensitive, due to the converter convert.exe!
						
						missile->m_mtx = m; // for better accuracy, we should also concider network latency and advance the missile as far as we think it has already travelled
						
						Vec3f pos, lookVec, upVec;
						missile->getPositionAndOrientation(pos, lookVec, upVec);
						missile->m_speedVec = lookVec; 
						
						m_missiles.insert(Missilemap::value_type(missileID, missile));
					}
				}
			}
			else if ( strcmp(head, "playerdeadbymissile") == 0 ) // enemy has been killed
			{
				int enemyID = -1;
				int missileID = -1;
				if(sscanf(message, "%s %d %d", head, &enemyID, &missileID) ) // read id-part of message
				{
					Missilemap::iterator mit = m_missiles.find(missileID);
					if(mit != m_missiles.end())
					{
						if(mit->second->m_status == Object::NORMAL)	// do not mark dead missile as exploding
						{
							mit->second->m_status = Object::EXPLODING;
							// increase score for owner of the missile
							Playermap::iterator owner = m_players.find(mit->second->m_ownerID);
							if(owner != m_players.end())
								owner->second->m_score++;
						}							
					}

					Playermap::iterator it = m_players.find(enemyID);
					if(it != m_players.end())
					{
						if ( (it->second->m_status != Object::DEAD) && 
							 (it->second->m_killerID != missileID) ) // dont die twice by the same missile 
						{
							if(it->second->m_status != Object::EXPLODING)
							{
								// play sound and only do it if it is not already exploding! 
								m_explSound.play();
							}
							
							it->second->m_killerID = missileID;
							it->second->m_status = Object::EXPLODING;

						}
					}
					
				}
			}				
		}
	}
}

void Game::writeDataToString(char* str, const void* data, int len)
{
	// Mind the fact that normal copying often uses word alignment
	// This would not work for messages between operating systems that use different byte-order (HiLo/LoHi)
	for(int i=0; i<len; i++)
	{
		str[i] = (((const char*) data)[i]);
	}
}
void Game::readDataFromString(const char* str, void* data, int len)
{
	// Mind the fact that normal copying often uses word alignment
	// This would not work for messages between operating systems that use different byte-order (HiLo/LoHi)
	for(int i=0; i<len; i++)
	{
		(((char*) data)[i]) = str[i];
	}
}

void Game::sendOurNewStatus()
{
	char msg[1024];
	Playermap::iterator it = m_players.find(m_playerID);	// look up ourselves
	Mtx4f& m = it->second->m_mtx;

	sprintf(msg, "p %d", m_playerID);
	int len = strlen(msg);
	writeDataToString(msg+len+1, &m, sizeof(Mtx4f));		// copy matrix into message
	m_socket.sendMessage(msg, len+1+sizeof(Mtx4f));
}

void Game::collisionDetection()
{
	// Test ourselves against all missiles
	// If collision, then send message that we died and by which missile
	// Also, set status to EXPLODING for ourselves and the missile 
	
	Playermap::iterator ourselves = m_players.find(m_playerID); 
	assert(ourselves != m_players.end());
	Player* me = ourselves->second;
	
	Missilemap::iterator it; 
	
	for(it = m_missiles.begin(); it != m_missiles.end(); it++)
	{
		//		if( ((it->second->m_status != Object::EXPLODING) || (me->m_status != Object::EXPLODING)) // fragments of us or missile can kill other missiles
		//			&& it->second->isColliding(*me))
		if (me->m_status != Object::DEAD && it->second->m_status != Object::DEAD) 
		{
			if(it->second->isColliding(*me))
			{
				// If both we and this missile already are exploding, then there would be no point in testing for collision again.
				// However, the UDP-protocol does not guarantee that the messages gets delivered.
				// Therefore, we keep sending colliding info a couple of times. E.g. as long as we are colliding.
				
				if (me->m_status == Object::NORMAL) {
					// play sound, but only if we are not already exploding
					m_explSound.play();
				}
				else if(it->second->m_status == Object::NORMAL) {
					m_explSound.play();	// play sound when missile explodes as well
				}
				if (me->m_status == Object::NORMAL) 
				{
					// Send message that we died and by which missile
					// For consistency between all players it is actually very important that this 
					// message "playerdeadbymissile" gets delivered, and preferrably  we should 
					// send it using TCP instead of UDP. Instead, we trust Chalmers network. 
					// The other types of messages are not so important if they get delivered or not.
	
					int missileID = it->first;			
					char msg[1024];
					sprintf(msg, "playerdeadbymissile %d %d", m_playerID, missileID);
					m_socket.sendMessage(msg, strlen(msg)+1);
				}
				me->m_status = Object::EXPLODING;	
				it->second->m_status = Object::EXPLODING;
			}
		}
	}
}

void Game::printToScreen(char* str, int x, int y, Vec3f color)
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	int w = glutGet((GLenum)GLUT_WINDOW_WIDTH);
	int h = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0, (w-1), 0, h-1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();   
	
	glColor3fv(color.vec);
	glRasterPos2f(x, y);

	int len=strlen(str);
	for(int i=0; i<len; i++)
		glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24, str[i]);
	
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	
	glPopAttrib();
}

void Game::printFrameRate()
{
	char str[200];
	static Timer tmr;
	
	float fps = 1.0 / tmr.elapsed();
	
	sprintf(str,"%2.2f fps",fps);
	printToScreen(str, 10, 10, Vec3f(1,0,0));	
	
	tmr.start(true);
}

void Game::printScores()
{
	Playermap::iterator it = m_players.find(m_playerID); 
	assert(it != m_players.end());
	Player* me = it->second;
	
	char str[200];
	sprintf(str,"%d", me->m_score);
	printToScreen(str, 490, 20, Vec3f(1,1,0));

}

void Game::setupLights()
{
	// setup light
	float light0Position[] = {0.0f, 200.0f, 0.0f, 1.0f};
	float light0Ambient[] =  {0.5f, 0.5f, 0.5f, 1.0f};
	float light0Diffuse[] =  {1, 1, 1, 1.0f};
	float light0Specular[] =  {0.25f, 0.25f, 0.25f, 1.0f};
	
	glLightfv(GL_LIGHT0, GL_POSITION, light0Position);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light0Ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light0Diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light0Specular);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	
	glEnable(GL_LIGHTING);  /* enable lighting */
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, 1);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, 1);
	glEnable(GL_NORMALIZE);
	glEnable(GL_DEPTH_TEST);
}

void Game::drawFloor(void)
{
	GLfloat black[4]={0,0,0,1.0};
	GLfloat grey[4]={0.6,0.6,0.6,1.0};
	GLfloat darkgrey[4]={0.2,0.2,0.2,1.0};
	
	/* enable texturing and set floorTexture to active */
	CHECK_ERROR;
	glColor3fv(grey);  // Used when lighting is disabled
	
	// Material is used when lighting is enabled
	glMaterialfv(GL_FRONT, GL_AMBIENT, darkgrey);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, grey);
	glMaterialfv(GL_FRONT, GL_SPECULAR, black);
	glMaterialfv(GL_FRONT, GL_EMISSION, black);
	glMateriali(GL_FRONT, GL_SHININESS, 0); // No shininess
	
	float s = 100;
	
	glBegin(GL_POLYGON);
	glNormal3f(0,1,0);
	glVertex3f( 1.0*s,0, 1.0*s);
	glVertex3f( 1.0*s,0,-1.0*s);
	glVertex3f(-1.0*s,0,-1.0*s);
	glVertex3f(-1.0*s,0, 1.0*s);
	glEnd();
	
	CHECK_ERROR;
}

void Game::drawCursor()
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	int w = glutGet((GLenum)GLUT_WINDOW_WIDTH);
	int h = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0, (w-1), 0, h-1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();   
	
	glColor3f(1,1,1);
	
	// Draw a cross here:




	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	
	glPopAttrib();
}

bool Game::windowHasFocus()
{
	return m_bWindowHasFocus;
}

void Game::getMouse()
{
	int delta_x;
	int delta_y;
	
	int w = glutGet((GLenum)GLUT_WINDOW_WIDTH);
	int h = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
	int x = glutGet((GLenum)GLUT_WINDOW_X);
	int y = glutGet((GLenum)GLUT_WINDOW_Y);
	
	POINT point;
	GetCursorPos(&point);
	if(false)
	{
		point.x -= x;
		point.y -= y;
		//		if(bWindowHasFocus)
		//			SetCursorPos(x + w/2, y + h/2);	// This is in full screen coordinates and may be outside the window - but that does not matter since we are only interested in relative motion in this application
		delta_x = point.x - w/2;
		delta_y = point.y - h/2;
	}
	else
	{
		delta_x = point.x - m_prev_x;
		delta_y = point.y - m_prev_y;
		m_prev_x = point.x;
		m_prev_y = point.y;
	}
	m_mouseDeltaX = delta_x;			
	m_mouseDeltaY = delta_y;			
}

void Game::setMouse()
{
	if(windowHasFocus())
	{
		int w = glutGet((GLenum)GLUT_WINDOW_WIDTH);
		int h = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
		int x = glutGet((GLenum)GLUT_WINDOW_X);
		int y = glutGet((GLenum)GLUT_WINDOW_Y);
		
		int xmin = x + 10;
		int ymin = y + 10;
		int xmax = x + w - 10;
		int ymax = y + w - 10;

		POINT point;
		GetCursorPos(&point);
		if (point.x < xmin || point.y < ymin || point.x > xmax || point.y > ymax) 
		{
			SetCursorPos(x + w/2, y + h/2);	
			m_prev_x = x + w/2;
			m_prev_y = y + h/2;
		}
	}
}
