#ifndef GAME_SPACEWAR_H
#define GAME_SPACEWAR_H

#include "object.h"
//#include "sockets.h"
#include "multicastsocket/multicastsocket.h"
#include "timer.h"
#include "missile.h"
#include "player.h"
#include "sound/bassound.h"
#include "navigation.h"
#include "camera.h"
#include "nature.h"


#include <vector>
#include <map>

class Game
{
public:
	Game();
	~Game();

	void InitGame();

	void renderFrame();
	
	int	 m_viewportWidth, m_viewportHeight;

	enum {FREE_NAV, LOCKED_NAV}; // LOCKED_NAV indicates that camera is locked to player's view. FREE_NAV gives overview look 
	int	 m_navigationMode;	

	bool windowHasFocus();
	bool m_bWindowHasFocus; // for debugging purposes it is good to avoid user input when our window does not have focus.
	int	 m_prev_x;			// These store last mouse position in order to compute delta movements
	int	 m_prev_y;
	
private:
	void updateFrame();

	void synchronizeStatus();
	
	void collisionDetection();
	
	void checkInput();

	void updateMissiles();
	
	void sendOurNewStatus();
	
	void updatePlayers();

	void initRemotePlayer(unsigned int playerID);
	
	void renderObjects();
	
	void printFrameRate();
	void printToScreen(char* str, int x, int y, Vec3f color);
	void printScores();
	void drawFloor();
	void drawCursor();
		
	void setCamera();
	void getMouse();
	void setMouse();
	
	void setupLights();
		

	int getUniqueID();

	void writeDataToString(char* str, const void* data, int len);
	void readDataFromString(const char* str, void* data, int len);
	
	// To avoid the need of a dedicated server we do as follows:
	// Each player selects a unique random number as its identifier 
	// In theory, two player could choose the same random number. 
	// In practice that will not happen. If this still happens, such cases can be detected and corrected.
	// We need a mapping between unique random playerID and the corresponding object.
	// We could easily change the random number to a name string instead.

	unsigned int		m_playerID; // our player ID
	
	typedef std::map<int, Object*>  Objectmap;
	typedef std::map<int, Player*>	Playermap;
	typedef std::map<int, Missile*> Missilemap;
	Playermap			m_players;  // <unique random playerID, player object>
	Missilemap			m_missiles; // <unique random missileID, missile object>

	CMulticastSocket	m_socket;

	double				m_timeLastShot; // the time the last shot was fired
	Timer				m_timer;

	CNavigation			m_navigator;
	Camera				m_Camera;
	int					m_mouseDeltaX;
	int					m_mouseDeltaY;
	double				m_timeLastInputCheck; // we need to know how long time has gone since the last input check in order to advance camera framerate independent
	
	CNature				m_nature;

	CBassSound			m_fireSound;	// sound when firing a weapon	
	CBassSound			m_explSound;	// sound of explosion
	CBassSound			m_bgSong;		// background song 
};

#endif