#ifndef L3DSRENDERER_H
#define L3DSRENDERER_H

#include "l3ds.h"
#include "boundingbox.h"
#include "collisiondetection.h"
#include <vector>

// Created by Ulf Assarsson 2005-02-16
class L3DSRenderer : public L3DS
{
public:
	// Default constructor
	L3DSRenderer() : 
	  m_bInitiated(false), 
	  m_bUseVertexBufferObjects(false) 
	  { m_transform.loadIdentity(); }; // display lists are often alot faster than explicit vbo's

	// Constuctor given a L3DS-object
	L3DSRenderer(L3DS& l3ds) : 
	  m_bInitiated(false), 
	  m_bUseVertexBufferObjects(false) 
	  {
		  *(L3DS*)this = l3ds;
		  m_transform.loadIdentity();
	  }; // display lists are often alot faster than explicit vbo's


	virtual ~L3DSRenderer();

	bool Load(const char* path, const char* filename);
	void RenderOGL(bool backfaceCulling, bool bOnlyGeo=false); // bOnlyGeo = true does not apply colors or textures
	void CreateDisplayList(unsigned int list, unsigned int mode, bool bOnlyGeo);
	void RenderObject(bool bOnlyGeo);	// this is lower level than RenderOGL()
	void init();
	void SetLights();
	const BoundingBox& GetBBox() {if(!m_bInitiated) init(); return m_bBox; }
	

	const Mtx4f& getMatrix() const { return m_transform; } 
	Mtx4f& getMatrix() { return m_transform; }

	const BV*	getBVH() const { return m_bvh; }
	BV*	getBVH() { return m_bvh; }

private:

	bool				m_bInitiated;
	bool				m_bUseVertexBufferObjects;
	Mtx4f				m_transform; 
	BoundingBox			m_bBox;
	BV*					m_bvh;			  // bounding volume hierarchy
	
	std::vector<int>	m_materialGroups; // list of number of consequtive triangles with same material id
	std::vector<unsigned int>	m_vboID;  // Array id
	std::vector<unsigned int>	m_elID;	  // Element id
};

#endif