/*
 * $Author: niklas $
 * $Date: 2004/07/05 17:36:44 $
 * $Log: timer.h,v $
 * Revision 1.1.1.1  2004/07/05 17:36:44  niklas
 * no message
 *
 * Revision 1.4.2.4  2004/05/12 09:48:30  jacob
 * For Mac
 *
 * Revision 1.4.2.3  2004/03/23 17:18:21  mats
 * Linux fixes
 *
*/

#ifndef TIMER_H
#define TIMER_H

#ifdef LINUX
#ifndef __ICC
typedef long long int __int64;
#endif
#endif

#include <string>
#include <stdio.h>

__int64 getCycleCount();

#ifdef LINUX
__int64 inline getCycleCount() {
#ifndef OSMac_
  unsigned int r1;
  unsigned int r2;

  __asm__ __volatile__("rdtsc;"
      : "=a"(r1), "=d"(r2));

  return ((__int64)r2)<<32 | r1;
#else
	return 0;
#endif
}
#endif

class Timer
{
public:
	Timer();
	Timer(const Timer& timer);
	virtual ~Timer();

public:
	inline void start(bool reset=false);
	inline void stop();
	inline double elapsed();
	inline std::string getTimeStr();
private:
	void init();
	void calculateFrequency();

private:
	__int64 m_start;
	__int64 m_adjust;
	__int64 m_frequency;
};


inline void Timer::start(bool reset)
{
	if ((!reset) && (m_start < 0))
		m_start += getCycleCount(); // we are starting with an accumulated time
	else
		m_start = getCycleCount();  // starting from zero
}

inline void Timer::stop()
{
	__int64 c = getCycleCount();

	if (m_start <= 0)
	{
		return; // was not running
	}
	
	m_start += -c; // stopped timer keeps elapsed timer ticks as a negative 

	if (m_adjust == 0)
	  return;

	// adjust for time timer code takes to run, but do not overflow
	if (m_start < m_adjust)
		m_start -= m_adjust;
	else 
		m_start = 0;
}

inline double Timer::elapsed()
{
	Timer result(*this);
	result.stop();
	return (double)(-result.m_start) / (double)m_frequency;
}

inline std::string Timer::getTimeStr()
{
	char timestr[100];
	double sec = elapsed();
	int h = (int)(sec / 3600.0);
	int m = (int)(sec / 60.0 - h*60.0);
	int s = (int)(sec - h*3600.0 - m*60.0);
	int ms = (int)((sec - int(sec)) * 100);
	sprintf(timestr, "%2.2d:%2.2d:%2.2d:%2.2d", h, m, s, ms);
	return timestr;
}


#endif
