#pragma warning (disable:4786)
#include <windows.h>
#include <GL/GLee.h>
#include <gl/glut.h>
#include <stdio.h>
#include <stdlib.h>

#include "game.h"
#include "navigation.h"

static Game game;

void changeSize(int w, int h) {
	// In the 2D-multiplayer game, the size of the playground had to remain the same for both players
	// However, in this 3D-game we can use the standard setting glViewport(0, 0, w, h);
	
	game.m_viewportWidth = w;
	game.m_viewportHeight = h;
	glViewport(0, 0, game.m_viewportWidth, game.m_viewportHeight);
}


void renderScene(void) 

{
	game.renderFrame();
}

void processNormalKeys(unsigned char key, int x, int y) {

	if (key == 27) 
		exit(0);

	if (key == 'n') // This toggles camera view between bird view and player's view
		game.m_navigationMode = (game.m_navigationMode==Game::FREE_NAV) ? Game::LOCKED_NAV: Game::FREE_NAV;

	if (key == 'l') // Choose some button to toggle loose of focus (good for debugging) 
		game.m_bWindowHasFocus = !game.m_bWindowHasFocus; 
}

void windowFocus(int state)
{
	// According to glut-spec, "state" indicates whether or not the mouse has left the window.
	// However, on Windows, it rather seems to represent whether or not our window has focus.
	
	if (state == GLUT_LEFT)
		game.m_bWindowHasFocus = false;
	if (state == GLUT_ENTERED)
	{
		game.m_bWindowHasFocus = true;
		int w = glutGet((GLenum)GLUT_WINDOW_WIDTH);
		int h = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
		int x = glutGet((GLenum)GLUT_WINDOW_X);
		int y = glutGet((GLenum)GLUT_WINDOW_Y);
		SetCursorPos(x + w/2, y + h/2);	
		game.m_prev_x = x + w/2;
		game.m_prev_y = y + h/2;
		glutSetCursor(GLUT_CURSOR_NONE); //GLUT_CURSOR_CROSSHAIR //GLUT_CURSOR_NONE
	}
}

int main(int argc, char **argv) {
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(game.m_viewportWidth, game.m_viewportHeight);
	glutCreateWindow("multiplayer3D");

	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);
	glutReshapeFunc(changeSize);
	glutKeyboardFunc(processNormalKeys);
	glutEntryFunc(windowFocus);

	game.InitGame();
	
	glutMainLoop();

	// just for compatibiliy purposes
	return 0;
}

