#ifndef MISSILE_H
#define MISSILE_H

#include "object.h"
#include <vector>

class CBassSound;

class Missile : public virtual Object // this virtual makes it possible for us to overload class methods
{
public:
	Missile();
	virtual ~Missile();

	virtual void render(Camera cam); // virtual means that this class overrides the Object::render()-method
	
	virtual void update(double currentTime, CNature* nature, CBassSound* explSound);

	static void loadExplosionTextures();
	
	Vec3f	m_speedVec; // speed vector of missile

private: 

	double m_timeOfBirth;					// the missiles have a maximum life time
	static double m_maxLifeTime;			// A missile only has ~3s life time
	static std::vector<int> m_explTexIDs;	// explosion texture IDs

private:
	static L3DSRenderer m_missileModel;		// stores the model data for the missile

	//---- ADDITIONALS FOR PARTICLE SYSTEM ----
	// Locally defined class for particle system
	class Particle
	{
	public:
		Vec3f m_pos;
		Vec3f m_speedVec;
	};

	void initParticles();

	std::vector<Particle>	m_particles;
	//------------------------------------------
};
#endif