
uniform sampler2D tex0, tex1;
varying vec4 vertex;

#if 1
void main()
{
	vec4 mossTexel  = texture2D(tex0, gl_TexCoord[0].xy);

	gl_FragColor.rgba = gl_Color.rgba; // This is interpolated lighting from the vertex shader
	
	gl_FragColor.rgba *= mossTexel;	  // This multiplies our green moss texture
	
	// Add white snow/grey stone on top of mountains and green color in the valleys
	vec4 green = vec4(0, 0.25, 0, 1);
	vec4 grey  = vec4(0.25, 0.25, 0.25, 1);
	float a = smoothstep(20.0, 30.0, vertex.y);		// a smoothly varies from 0 to 1 for vertex.y-values between 20 to 30.
	gl_FragColor.rgba += mix(green, grey, a);	// interpolate between green and grey/white with a as interpolation factor
}


#else

varying vec4 passcolor;
varying vec3 lightDir;

void main()
{
	vec4 mossTexel  = texture2D(tex0, gl_TexCoord[0].xy);

   //Get the color of the bump-map
   vec3 BumpNorm = vec3(texture2D(tex1, gl_TexCoord[1].xy));


    //Expand the bump-map from [0,1] to [-1,1] for each element xyz.
    BumpNorm = (BumpNorm -0.5) * 2.0;

    //Find the dot product between the light direction and the normal
    float NdotL = max(dot(BumpNorm, lightDir), 0.0);

    //Calculate the diffuse color 
    vec3 diffuse = NdotL * passcolor.xyz;

    //Set the color of the fragment...  If you want specular lighting or other types add it here
    gl_FragColor.rgba = vec4(diffuse, 1) * mossTexel;

	
	float a = smoothstep(10.0, 100.0, vertex.y);
	vec4 green = vec4(0, 0.2, 0, 1);
	vec4 grey  = vec4(0.65, 0.65, 0.65, 1);
	gl_FragColor.rgba += mix(green, grey, a);

}



#endif

