// minimal vertex shader

varying vec4 vertex;

#if 1
void main()
{	

	// The following three lines provide the same result
//	gl_Position = gl_ProjectionMatrix * gl_ModelViewMatrix * gl_Vertex;
//	gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
	gl_Position = ftransform();


	gl_TexCoord[0] = gl_MultiTexCoord0;	// just forward to fragment shader

	vertex = gl_Vertex;		// send world space coordinate of vertex to fragment shader

	// Compute standard shading. This is identical to what we did in lecture 10 - "Vertex and fragment shaders"

		vec3 normal, lightDir;
		vec4 diffuse, ambient, globalAmbient, specular;
		float NdotL;
		
		vec4 eyeVertex = gl_ModelViewMatrix * gl_Vertex;	// transform vertex into eye space
		normal = normalize(gl_NormalMatrix * gl_Normal);	// transform normal into eye space
		vec4 lightpos = gl_LightSource[0].position;			// according to spec, gl_LightSource[0].position is in eye space
		lightDir = normalize(lightpos.xyz-eyeVertex.xyz); 
		NdotL = max(dot(normal, lightDir), 0.0);
		diffuse = gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse;
		
		/* Compute the ambient and globalAmbient terms */
		ambient = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
		globalAmbient = gl_LightModel.ambient * gl_FrontMaterial.ambient;
		
		/* Compute specular term */
		if (NdotL > 0.0) 
		{
			// normalize the half-vector, and then compute the 
			// cosine (dot product) with the normal
			float NdotHV = max(dot(normal, gl_LightSource[0].halfVector.xyz),0.0);
			specular = gl_FrontMaterial.specular * gl_LightSource[0].specular * 
					pow(NdotHV,gl_FrontMaterial.shininess);
		}
		else
		{
			specular = vec4(0,0,0,0);
		}

		gl_FrontColor =  NdotL * diffuse + specular + globalAmbient + ambient;
}
#else

// Bump map version

varying vec4 passcolor;
varying vec3 lightDir;

void main()
{	
	gl_Position = ftransform();
	gl_TexCoord[0] = gl_MultiTexCoord0; // just forward to fragment shader
	gl_TexCoord[1] = gl_MultiTexCoord1; // just forward to fragment shader
	vertex = gl_Vertex;					// send world space coordinate of vertex to fragment shader


    //Put the color in a varying variable
    passcolor = gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse; //gl_Color;

	// OpenGL is not aware of our world space. OpenGL works in eye-space. 
	// Therefore, we do our computations in eye-space.
	vec3 normal = normalize(gl_NormalMatrix * gl_Normal);	// transform normal into eye space
	vec4 lightpos = gl_LightSource[0].position;				// gl_LightSource[0].position is already in eye space
	vec4 eyeVertex = gl_ModelViewMatrix * gl_Vertex;		// transform vertex into eye space
	lightDir = normalize(lightpos.xyz-eyeVertex.xyz);		// now, lightDir is in eye space
	lightDir = vec3(gl_ModelViewMatrix * vec4(0,1,0,0));	// Uncomment this line to disregard light position and get directional light. Less accurate, but looks better.

	// Transform lightDir from eye-space into bump-map space
	// Bump map space is defined with z-axis along undisturbed normal direction.
	// We have to choose two perpendicular vectors for x- and y-axis. We know 
	// that our height field happens to be in the xz-plane in world space. Thus 
	// we can choose:
	// bump space x = (1,0,0) cross normal
	// bump space y = n cross (bump space x)
	// bump space z = normal
	// 
	// Since we are working in eye space, we must transform (1,0,0) from world-space into eye-space

	vec3 xaxis = vec3(gl_ModelViewMatrix * vec4(1,0,0,0));
	vec3 transfLightDir, ex, ey, ez;
	ex = cross(xaxis, normal);
	ey = cross(normal, ex);
	ez = normal;
//	transfLightDir.x = dot(ex,lightDir);
//	transfLightDir.y = dot(ey,lightDir);
//	transfLightDir.z = dot(ez,lightDir);
//	lightDir = normalize(transfLightDir);

	// We would like the rotation matrix to have ex, ey, ez in 1:st, 2:nd and 3:rd row.
	// However, the constructor takes columns. Therefore we premultiply the vector with the matrix

	mat3 rotmat = mat3(ex, ey, ez);		// This creates the matrix columnwise, i.e., with ex along 1:st column etc
    lightDir = lightDir * rotmat;		// Rotate the light into tangent space.
	normalize(lightDir);

}


#endif