/*
	This code uses HawkNL, which is under the GPU-licence.
	Therefore, this code as well as all SpaceWar-code is under the GNU-licence.
	Some code is taken from multicast.c, as described just below	
	
	/Ulf Assarsson
*/
/*
  Sample program for the HawkNL cross platform network library
  Copyright (C) 2000-2002 Phil Frisbie, Jr. (phil@hawksoft.com)
*/
/*
  This sample app shows client and server Multicasting.

  For complete usage, use multicast -h
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "HawkNL1.70/include/nl.h"

#if defined WIN32 || defined WIN64

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#define sleep(x)    Sleep((DWORD)(1000 * (x)))
#else
#include <unistd.h>
#define stricmp strcasecmp
#endif

#include "multicastsocket.h"


//    printf("multicast_address full range: 224.0.0.0 to 239.255.255.255\n");
//    printf("            recomended range: 234.0.0.0 to 238.255.255.255\n");
//    printf("                     default: 234.235.236.237\n\n");
//    printf("port number: 1024 to 65535\n");
//    printf("    default: 1258\n\n");
//    printf("ttl value: 1 to 255\n");
//    printf("recomended values:   1 for local LAN\n");
//    printf("                    15 for a site\n");
//    printf("                    63 for a region\n");
//    printf("                   127 for world\n");
//    printf("          default: 1\n");
//    printf("DO NOT use a ttl higher than you need\n\n");
//    printf("interval: 1 to 60 seconds\n");
//    printf(" default: 10 seconds\n\n");
//

static void printErrorExit(void)
{
    NLenum err = nlGetError();

    if(err == NL_SYSTEM_ERROR)
    {
        printf("System error: %s\n", nlGetSystemErrorStr(nlGetSystemError()));
    }
    else
    {
        printf("HawkNL error: %s\n", nlGetErrorStr(err));
    }
    nlShutdown();
}

CMulticastSocket::~CMulticastSocket()
{
	nlClose(m_socket);
	nlShutdown();
}

void CMulticastSocket::sendMessage(const char* str, int len) // str should string of max 1024 bytes
{
    (void)nlWrite(m_socket, (NLvoid *)str, len);
}

const char* CMulticastSocket::receiveMessage()
{
	m_buffer[0]='\0';

	nlRead(m_socket, (NLvoid *)m_buffer, (NLint)sizeof(m_buffer));
	//NLaddress   addr;
	//nlGetRemoteAddr(sock, &addr);
	
	return m_buffer;
}

bool CMulticastSocket::connect(int port)
{
    NLaddress   addr, maddr;
    NLbyte      string[NL_MAX_STRING_LENGTH];
    NLbyte      address[] = "234.235.236.237";
//    NLushort    port = 1258;
    NLint       ttl = NL_TTL_LOCAL;
    NLaddress   *alladdr;
    NLint       count;

    if(nlInit() == NL_FALSE)
	{
        printErrorExit();
		return false;
	}

    if(nlSelectNetwork(NL_IP) == NL_FALSE)
	{
        printErrorExit();
		return false;
	}

	nlStringToAddr(address, &maddr);
	
    if(port < 1024 || port > 65535)
    {
		port = 3456;
        printf("error: invalid port, using default port of %u\n", port);
        nlSetAddrPort(&maddr, port);
    }
    else
    {
        nlSetAddrPort(&maddr, port);
    }


    /* change TTL if you want to get out of your LAN */
    /* note that this call must be before nlConnect */
    nlHint(NL_MULTICAST_TTL, ttl);
    /* enable reuse address to run two or more copies on one machine */
    nlHint(NL_REUSE_ADDRESS, NL_TRUE);
    /* list all the local addresses */
    printf("local addresses are:\n");
    alladdr = nlGetAllLocalAddr(&count);
    if(alladdr != NULL)
    {
        while(count-- > 0)
        {
            printf("  %s\n", nlAddrToString(alladdr++, string));
        }
    }
    printf("\n");


    /* create a socket */
    /* to just write, you can use port 0 */
    /* but if you want to read also, then use the */
    /* multicast port */
    m_socket = nlOpen(port, NL_UDP_MULTICAST);

    if(m_socket == NL_INVALID) 
	{
        printErrorExit();
		return false;
	}
	
    nlGetLocalAddr(m_socket, &addr);
    printf("our socket address is %s\n", nlAddrToString(&addr, string));

    if(nlConnect(m_socket, &maddr) == NL_FALSE)
    {
        nlClose(m_socket);
        printErrorExit();
		return false;
    }
    printf("multicast address %s\n", nlAddrToString(&maddr, string));

	return true;
}

