#pragma warning (disable:4786)

#include <windows.h>
#include <GL/GLee.h>
#include <gl/glut.h>
#include "nature.h"
#include "vecmath.h"
#include "fractal/Fractal.h"
#include "textfile.h"
#include "tex/texturemanager.h"
#include "object.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#ifndef M_PI
#define M_PI       3.14159265358979323846f
#endif

#define USE_BILLBOARDS	1

// THIS IS A MACRO THAT PRINTS ANY OPENGL ERROR TO THE COMMAND PROMT WINDOW
// USE IT ALOT IN YOUR CODE
static GLenum errCode;
#define CHECK_ERROR 	errCode = glGetError();	\
	if (errCode != GL_NO_ERROR) {	\
	printf("(%s, %d) ERROR: %s\n", __FILE__, __LINE__, gluErrorString(errCode)); \
	}

L3DSRenderer CNature::m_treeModel;		
int			 CNature::m_treeTexID = -1;


void CNature::renderMountain()
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	//glUseProgramObjectARB(m_mtProgram); 
	glCallList(m_mountainID);
	//glUseProgramObjectARB(0); 
	glPopAttrib();
	
}

void CNature::createMountain()
{

	bool bFirstTime=true;
	if (bFirstTime) 
	{
		bFirstTime = false;
		loadShaders();		

		Init_3DNoise(); 	// Initiate the noise table only once
	}


	CHECK_ERROR;
	glPushAttrib(GL_ALL_ATTRIB_BITS); // Do before display list start. Otherwise, it is compiled into display list, which slows down almost 10 times!!

	GLuint dirtTex = TextureManager::load("media/textures/nature/dirt_earth-n-moss_df_.tga");	// load green moss texture map
	GLuint bumpTex = TextureManager::load("media/textures/nature/bump_thickswirls_df_.tga");	// load bump map
	
	// We will create a display list for the full rendering of the fractal mountain. See void CNature::renderMountain() above.
	m_mountainID = glGenLists(1);	// get display list id
	assert(m_mountainID != 0);
	
	glNewList(m_mountainID, GL_COMPILE);	// tell OpenGL that we start the definition of a new display list. Most OpenGL-calls will be added until we call glEndList at end of this method.

	glEnable(GL_LIGHTING);
	glEnable(GL_CULL_FACE);

	GLfloat black[4]={0,0,0,1.0f};
	GLfloat grey[4]={0.8f,0.8f,0.8f,1.0f};
	GLfloat darkgrey[4]={0.2f,0.2f,0.2f,1.0f};
	
	glColor3fv(grey);  // Used if lighting would happen to be disabled
	
	// Material is used when lighting is enabled
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, darkgrey);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, grey);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, darkgrey);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, black);
	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 100); // No shininess
	
	//glUseProgramObjectARB(m_mtProgram); // HERE:

	// Set textures
	glActiveTexture(GL_TEXTURE0); // activate tex unit 0
	GLuint tex = dirtTex;
	glBindTexture(GL_TEXTURE_2D, tex);
	//glEnable(GL_TEXTURE_2D);		  // This is not necessary when we are using fragment shaders
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR); // this is nicest available mipmap interpolation
	glUniform1iARB(glGetUniformLocationARB( m_mtProgram,"tex0"),0);	

	glActiveTexture(GL_TEXTURE1); // activate tex unit 0
	tex = bumpTex;
	glBindTexture(GL_TEXTURE_2D, tex);
	//glEnable(GL_TEXTURE_2D);		  // This is not necessary when we are using fragment shaders
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR); // this is nicest available mipmap interpolation
	glUniform1iARB(glGetUniformLocationARB( m_mtProgram,"tex1"),1);	
	
	// Create the mesh for the fractal landscape
	m_offset = Vec3f(-100, 0, -100)*2.0f;
	m_scale  = Vec3f(2, 50, 2)*2.0f;
	int i;
	for(i=0; i<100; i++)
	{
		for(int j=0; j<100; j++)
		{
			Vec3d v(j, 0, i);
			double h = RidgedMultifractal( Vec3d(j*0.022, 0, i*0.022) , 1.3, 2.48, 5, 1, 2 ); // 0.9, 2.48, 5, 1, 2 
			h = 0;	// HERE:
			m_mountainMesh[i][j] = Vec3f(j, h, i).mult(m_scale) + m_offset;
		}	
	}

	// Compute normals and texture coords for each vertex. 
	// Normal is average of surrounding triangle normals
	// We use the method in Game Programming Gems III, page 344
	Vec3f normals[100][100];
	Vec2f texCoords[100][100];
	for(i=0; i<100; i++)
	{
		for(int j=0; j<100; j++)
		{
			float h  = m_mountainMesh[i][j][1];
			float h4 = (i>=1)  ? m_mountainMesh[i-1][j][1] : h;
			float h2 = (i<=98) ? m_mountainMesh[i+1][j][1] : h;
			float h3 = (j>=1)  ? m_mountainMesh[i][j-1][1] : h;
			float h1 = (j<=98) ? m_mountainMesh[i][j+1][1] : h;

			Vec3f n(h3-h1, 2, h4-h2);
			n.normalize();

			normals[i][j] = n;

			texCoords[i][j] = Vec2f(j,i);
		}	
	}
	
	const float ts = 0.09f;	// ground texture scale
	const float bs = 0.01f;	// bump texture scale
	
	// Render the mesh (or actually create the display list)
	int version = 2;
	switch(version) {
	case 1:
		glBegin(GL_QUADS);
		for(i=0; i<99; i++)
		{
			for(int j=0; j<99; j++)
			{
				Vec3f v[4];
				v[0] = m_mountainMesh[i+1][j];
				v[1] = m_mountainMesh[i+1][j+1];
				v[2] = m_mountainMesh[i][j+1];
				v[3] = m_mountainMesh[i][j];
				
				Vec3f n[4];
				n[0] = normals[i+1][j];
				n[1] = normals[i+1][j+1];
				n[2] = normals[i][j+1];
				n[3] = normals[i][j];
				
				Vec2f t[4];
				t[0] = texCoords[i+1][j];
				t[1] = texCoords[i+1][j+1];
				t[2] = texCoords[i][j+1];
				t[3] = texCoords[i][j];
				
				glNormal3fv	 (n[0].vec);
				glMultiTexCoord2fv(GL_TEXTURE0, (t[0]*ts).vec);
				glMultiTexCoord2fv(GL_TEXTURE1, (t[0]*bs).vec);
				glVertex3fv	 (v[0].vec);
				
				glNormal3fv  (n[1].vec);
				glMultiTexCoord2fv(GL_TEXTURE0, (t[1]*ts).vec);
				glMultiTexCoord2fv(GL_TEXTURE1, (t[1]*bs).vec);
				//glTexCoord2fv(t[1].vec);
				glVertex3fv  (v[1].vec);
				
				glNormal3fv  (n[2].vec);
				glMultiTexCoord2fv(GL_TEXTURE0, (t[2]*ts).vec);
				glMultiTexCoord2fv(GL_TEXTURE1, (t[2]*ts).vec);
				//glTexCoord2fv(t[2].vec);
				glVertex3fv  (v[2].vec);
				
				glNormal3fv  (n[3].vec);
				glMultiTexCoord2fv(GL_TEXTURE0, (t[3]*ts).vec);
				glMultiTexCoord2fv(GL_TEXTURE1, (t[3]*bs).vec);
				//glTexCoord2fv(t[3].vec);
				glVertex3fv  (v[3].vec);
				
			}	
		}
		glEnd();
		break;
	case 2:
		glBegin(GL_TRIANGLES);
		for(i=0; i<99; i++)
		{
			for(int j=0; j<99; j++)
			{
				Vec3f v[4];
				v[0] = m_mountainMesh[i+1][j];
				v[1] = m_mountainMesh[i+1][j+1];
				v[2] = m_mountainMesh[i][j+1];
				v[3] = m_mountainMesh[i][j];
				
				Vec3f n[4];
				n[0] = normals[i+1][j];
				n[1] = normals[i+1][j+1];
				n[2] = normals[i][j+1];
				n[3] = normals[i][j];
				
				Vec2f t[4];
				t[0] = texCoords[i+1][j];
				t[1] = texCoords[i+1][j+1];
				t[2] = texCoords[i][j+1];
				t[3] = texCoords[i][j];

				// find the split diagonal of the quad that gives the smallest angle
				// between the two resulting triangle normals

				// case 1, split along diagonal v0-v2
				Vec3f n1 = (v[1]-v[0]).cross(v[2]-v[0]);
				n1.normalize();
				Vec3f n2 = (v[2]-v[0]).cross(v[3]-v[0]);
				n2.normalize();
				float cosangle1 = n1.dot(n2);

				// case 2, split along diagonal v1-v3
				n1 = (v[2]-v[1]).cross(v[3]-v[1]);
				n1.normalize();
				n2 = (v[3]-v[1]).cross(v[0]-v[1]);
				n2.normalize();
				float cosangle2 = n1.dot(n2);
				
				if(cosangle1 > cosangle2)
				{	// choose case 1
					glNormal3fv  (n[0].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[0]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[0]*bs).vec);
					glVertex3fv  (v[0].vec);
					
					glNormal3fv  (n[1].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[1]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[1]*bs).vec);
					glVertex3fv  (v[1].vec);
					
					glNormal3fv  (n[2].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[2]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[2]*bs).vec);
					glVertex3fv  (v[2].vec);
					//--
					glNormal3fv  (n[0].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[0]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[0]*bs).vec);
					glVertex3fv  (v[0].vec);

					glNormal3fv  (n[2].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[2]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[2]*bs).vec);
					glVertex3fv  (v[2].vec);

					glNormal3fv  (n[3].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[3]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[3]*bs).vec);
					glVertex3fv  (v[3].vec);
				}
				else
				{
					glNormal3fv  (n[0].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[0]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[0]*bs).vec);
					glVertex3fv  (v[0].vec);
					
					glNormal3fv  (n[1].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[1]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[1]*bs).vec);
					glVertex3fv  (v[1].vec);
					
					glNormal3fv  (n[3].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[3]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[3]*bs).vec);
					glVertex3fv  (v[3].vec);
					//					
					glNormal3fv  (n[1].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[1]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[1]*bs).vec);
					glVertex3fv  (v[1].vec);
					
					glNormal3fv  (n[2].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[2]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[2]*bs).vec);
					glVertex3fv  (v[2].vec);
					
					glNormal3fv  (n[3].vec);
					glMultiTexCoord2fv(GL_TEXTURE0, (t[3]*ts).vec);
					glMultiTexCoord2fv(GL_TEXTURE1, (t[3]*bs).vec);
					glVertex3fv  (v[3].vec);
				}
			}	
		}
		glEnd();
		
		break;
	default:
		;
	}
	
	// Disable vertex- and fragment shaders
	glUseProgramObjectARB(0); 
	
	glEndList();
	
	glPopAttrib();
	CHECK_ERROR;
		
}


void CNature::loadShaders() 
{
	char *vs = NULL,*fs = NULL;

	m_mtVertShader = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	m_mtFragShader = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
	fs = textFileRead("mountain.frag");
	vs = textFileRead("mountain.vert");
	
	const char * vv = vs;
	const char * ff = fs;
	glShaderSourceARB(m_mtVertShader, 1, &vv,NULL);
	glShaderSourceARB(m_mtFragShader, 1, &ff,NULL);
	free(vs);free(fs);
	
	glCompileShaderARB(m_mtVertShader);
	glCompileShaderARB(m_mtFragShader);
	printInfoLog(m_mtVertShader);
	printInfoLog(m_mtFragShader);
	
	m_mtProgram = glCreateProgramObjectARB();
	glAttachObjectARB(m_mtProgram, m_mtVertShader);
	glAttachObjectARB(m_mtProgram, m_mtFragShader);
	
	glLinkProgramARB(m_mtProgram);
	printInfoLog(m_mtProgram);
}

float CNature::getHeight(float x, float z)
{
	// Returns height at given position in the height field. 
	// The height field is oriented in the xz-plane

	// Interpolate the height from the 4 surrounding mesh vertices
	x = (x-m_offset[0]) / m_scale[0];
	z = (z-m_offset[2]) / m_scale[2];


	int x0 = x; // truncated 
	int x1 = x0+1;
	int z0 = z; // truncated
	int z1 = z0+1;

	// check boundaries
	x0 = max(0, x0);
	x1 = max(0, x1);
	z0 = max(0, z0);
	z1 = max(0, z1);

	x0 = min(99, x0);
	x1 = min(99, x1);
	z0 = min(99, z0);
	z1 = min(99, z1);

	float h0 = m_mountainMesh[z0][x0][1];
	float h1 = m_mountainMesh[z0][x1][1];
	float h2 = m_mountainMesh[z1][x1][1];
	float h3 = m_mountainMesh[z1][x0][1];

	// interpolate in x-direction
	float ax = x-x0;
	float h01 = h1*ax + h0*(1-ax);
	float h23 = h2*ax + h3*(1-ax);

	// interpolate in z-direction
	float az = z-z0;
	float height = h23*az + h01*(1-az);
	
	return height;
}


void CNature::createTrees()
{
#if !USE_BILLBOARDS
	static bool bFirstTime = true;
	if(bFirstTime)
	{
		// Load the tree model
		m_treeModel.LoadFile("media/models/tree.3ds"); // Loads 3ds-model
		Mtx4f m0, m1, m2, m3;
		m0.loadIdentity();
		m1.loadIdentity();
		m2.loadIdentity();
		m3.loadIdentity();
		m1.scale(10, 10, 10);
		m2.rotX(M_PI);
		Vec3f center = m_treeModel.GetBBox().centerPoint();
		m0.translate(0, 0, 0);
		m_treeModel.getMatrix() = m3*m2*m1*m0; // translate to center, scale by 0.01 and rotate -90 degrees around x-axis
	}

	for(int i=0; i<10; i++)
	{
		
		// find random position in terrain, below certain height
		Vec3f pos;
		do 
		{
			float x = rand()/(float)RAND_MAX;
			float z = rand()/(float)RAND_MAX;

			pos = Vec3f(x, 0, z).mult(m_scale)*100.0f + m_offset;
			pos[1] = getHeight(pos[0], pos[2]);
			
		} while(pos[1] > 40);
		

		Object* tree = new Object;

		tree->m_l3dsObj = &m_treeModel;
		tree->m_useBackFaceCulling = false;

		Vec3f hv = m_treeModel.GetBBox().halfVector();
		hv = m_treeModel.getMatrix().multVec(hv);

		pos[1] -= hv[1];
		// face center of the world
		Vec3f lookVec = Vec3f(rand()/(float)RAND_MAX, 0, rand()/(float)RAND_MAX); 
		lookVec.normalize();
		
		Vec3f upVec(0,1,0);
		tree->setPositionAndOrientation(pos, lookVec, upVec);
		
		m_trees.push_back(tree);	
	}
#else
	for(int i=0; i<10; i++)
	{
		Vec3f pos; // find random position in terrain, below certain height
		do {
			pos = Vec3f(rand()/(float)RAND_MAX, 0, rand()/(float)RAND_MAX).mult(m_scale)*100.0f + m_offset;
			pos[1] = getHeight(pos[0], pos[2]);
		} while(pos[1] > 40);
		m_treePositions.push_back(pos);
	}
	
	// Load billboard texture
	unsigned char transp[] = {0,0,0};
	if (m_treeTexID == -1) 
		m_treeTexID = TextureManager::load("media/textures/nature/treeBillboard.tif", transp, false);
	
#endif
}

void CNature::renderTrees(Camera cam)
{
#if !USE_BILLBOARDS
	// Add one light from below for the trees to look better
	float light0Position[] = {0.0f, -200.0f, 0.0f, 1.0f};
	float light0Ambient[] =  {0.5f, 0.5f, 0.5f, 1.0f};
	float light0Diffuse[] =  {1, 1, 1, 1.0f};
	float light0Specular[] =  {0.25f, 0.25f, 0.25f, 1.0f};
	
	glLightfv(GL_LIGHT1, GL_POSITION, light0Position);
	glLightfv(GL_LIGHT1, GL_AMBIENT, light0Ambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, light0Diffuse);
	glLightfv(GL_LIGHT1, GL_SPECULAR, light0Specular);
	glEnable(GL_LIGHT1);

	
	for(int i=0; i<m_trees.size(); i++)
		m_trees[i]->render(cam);

	glDisable(GL_LIGHT1); // restore to just one light

#else
	glPushAttrib(GL_ALL_ATTRIB_BITS); // save current OpenGL state
	
	// Draw a billboard for each tree.
	
	//	glActiveTexture(GL_TEXTURE0 + 0); 
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, m_treeTexID);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	
	glDisable(GL_CULL_FACE);
	//glDepthMask(0); // disable updating of z-buffer
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE, GL_REPLACE); //GL_MODULATE
	glEnable(GL_BLEND);
	glColor4f(1,1,1,1);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_ALPHA_TEST);
	glAlphaFunc(GL_GREATER, 0.1f);
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	for(unsigned int i=0; i<m_treePositions.size(); i++)
	{
		// Vec3f dim(5,5,5); //0.599445, 4.84506, -4.79991 );
		Vec3f c = m_treePositions[i];
		float r = 10;
		
		// Create a quad in screenX and screenY directions
		Vec3f lookVec = cam.m_lookAt - cam.m_lookFrom;
		lookVec.normalize();
		
		Vec3f screenXaxis = lookVec.cross(cam.m_upVec);
		screenXaxis.normalize();
		
		Vec3f up = screenXaxis.cross(lookVec);
		up.normalize();
		//up = Vec3f(0,1,0); // uncomment for axial billboarding
		
		Vec3f v0 = c - screenXaxis*0.5f*r; //c - up*0.5f*r - screenXaxis*0.5f*r;
		Vec3f v1 = v0 + screenXaxis*r;
		Vec3f v2 = v1 + up*r;
		Vec3f v3 = v0 + up*r;
		
		
		glBegin(GL_QUADS);
		glTexCoord2f(1,1);
		glVertex3fv(v0.vec);
		
		glTexCoord2f(0,1);
		glVertex3fv(v1.vec);
		
		glTexCoord2f(0,0);
		glVertex3fv(v2.vec);
		
		glTexCoord2f(1,0);
		glVertex3fv(v3.vec);
		glEnd();
		
	}

	glPopMatrix();
	
	glDisable(GL_BLEND);		// These 3 lines are actually not necessary since we restore full OpenGL-state below
	glDisable(GL_TEXTURE_2D);	
	glDepthMask(1);				// enable updating of z-buffer
	
	glPopAttrib();				// restore OpenGL-state

#endif
}