#include "windows.h"
#include "windowsx.h"
#include "navigation.h"
//#include <math.h>
//#include <stdlib.h>
#include <GL/GLee.h>
#include <gl/glut.h>


#ifndef M_PI
#define M_PI       3.14159265358979323846
#endif


void CNavigation::assignKeys()
{
	m_strafe_left	= 'A';
	m_strafe_right	= 'D';
	m_forward		= 'W';
	m_backward		= 'S';
	m_up			= 'R';
	m_down			= 'F';
	m_turn_left		= VK_LEFT;
	m_turn_right	= VK_RIGHT;
	m_turn_up		= VK_UP;
	m_turn_down		= VK_DOWN;

	// set alternatives to the same unless you have any better key suggestions
	m_alt_strafe_left	= 'A';
	m_alt_strafe_right	= 'D';
	m_alt_forward		= 'W';
	m_alt_backward		= 'S';
	m_alt_turn_left		= VK_LEFT;
	m_alt_turn_right	= VK_RIGHT;
	m_alt_turn_up		= VK_UP;
	m_alt_turn_down		= VK_DOWN;

	m_max_height	= 320;	// restricts how high and low the user can go.    
	m_min_height	= 0;
	m_lock_upvec	= true;

	m_speed = 2.0f;
	m_turnSpeed = (2*M_PI/100.0f);
	m_max_up_down_rot = (1.5f * M_PI*0.25f);

	m_upvec = Vec3f(0,1,0);
	m_lookAt = Vec3f(0,16,1); //Vec3f(0,16,-1);
	m_lookFrom = Vec3f(0,16,0);
}

void CNavigation::handleInput(int delta_x, int delta_y)
{

	//------------- QUAKE STYLE ---------------

	Vec3f lookVec = m_lookAt - m_lookFrom;
	lookVec.normalize();
	Vec3f screenXaxis = lookVec.cross(m_upvec);
	screenXaxis.normalize();

	Vec3f up = screenXaxis.cross(lookVec);
	up.normalize();

	m_upvec.normalize();

	Vec3f forward = lookVec - m_upvec*(lookVec.dot(m_upvec));
	forward.normalize();

	float turnSpeed = m_turnSpeed;
	float speed = m_speed;


	//--- HANDLE MOUSE INPUT ---
	// Keyboard input is easily handled on the fly below


	{
		static bool bFirstTime=true;

		
		if( true ) //GetAsyncKeyState(VK_LBUTTON) & 0x8000 ) // uncomment if you only want to navigate if left mouse button is pressed (user's choise)
		{
			float turnAngleH = delta_x * 0.3 * M_PI/180.0;
			float turnAngleV = delta_y * 0.3 * M_PI/180.0;

			if(bFirstTime)
			{
				bFirstTime=false;
			}
			else
			{
				// Turn left/right
				// rotate lookat around upvec
				{
					Vec3f lookVector = m_lookAt - m_lookFrom;
					Mtx3f m;
					m.rotAxis(m_upvec, -turnAngleH); 
					m_lookAt = m_lookFrom + m * lookVector; // rotate lookVector around up-vector
				}

				// update screenXaxis and lookvec
				lookVec = m_lookAt - m_lookFrom;
				lookVec.normalize();
				screenXaxis = lookVec.cross(m_upvec);
				screenXaxis.normalize();
				forward = lookVec - m_upvec*(lookVec.dot(m_upvec));
				forward.normalize();

				// Turn up/down
				{
					// rotate lookat around screenXaxis
					Vec3f lookVector = m_lookAt - m_lookFrom;
					// rotation is max 1.5*pi/4
					float turnAngle = fabs(turnAngleV); //turnSpeed;
					float cosangle = lookVec.dot(forward);
					float angle = acos(cosangle); // acos returns radians

					if(turnAngleV > 0) // flip this if flipped vertical mouse navigation
					{	// down
						if(m_upvec.dot(lookVec) <= 0) // current angle is positive downwards
						{
							if(angle + turnAngle > m_max_up_down_rot)
								turnAngle = m_max_up_down_rot - angle;
						}
						turnAngle = -turnAngle;
					}
					else
					{	// up
						if(m_upvec.dot(lookVec) >= 0) // current angle is positive downwards
						{
							if(angle + turnAngle > m_max_up_down_rot)
								turnAngle = m_max_up_down_rot - angle;
						}
					}
					Mtx3f m;
					m.rotAxis(screenXaxis, turnAngle); 
					m_lookAt = m_lookFrom + m * lookVector; // rotate lookVector around up-vector
				}
			}
		}
		else
			bFirstTime = true;
	}



	// update screenXaxis and lookvec
	lookVec = m_lookAt - m_lookFrom;
	lookVec.normalize();
	screenXaxis = lookVec.cross(m_upvec);
	screenXaxis.normalize();
	forward = lookVec - m_upvec*(lookVec.dot(m_upvec));
	forward.normalize();


	//--- STRIFE ---
	if( (GetAsyncKeyState(CNavigation::m_strafe_left) &0x8000) ||
		(GetAsyncKeyState(CNavigation::m_alt_strafe_left) &0x8000) )
	{
		m_lookAt -= screenXaxis*speed;
		m_lookFrom -= screenXaxis*speed;
	}
	if( (GetAsyncKeyState(CNavigation::m_strafe_right) &0x8000) ||
		(GetAsyncKeyState(CNavigation::m_alt_strafe_right) &0x8000) )
	{
		m_lookAt += screenXaxis*speed;
		m_lookFrom += screenXaxis*speed;
	}
	if( (GetAsyncKeyState(CNavigation::m_forward) &0x8000) )
	{
		m_lookAt += forward*speed;
		m_lookFrom += forward*speed;
	}
	if( (GetAsyncKeyState(CNavigation::m_backward) &0x8000) )
	{
		m_lookAt -= forward*speed;
		m_lookFrom -= forward*speed;
	}
	if( (GetAsyncKeyState(CNavigation::m_up) &0x8000) )
	{
		// check height above plane through origin
		// plane equation: n + d = 0;
		// n=m_upvec, d=-n dot (0,0,0);
		float height = m_lookFrom.dot(m_upvec);
		float rizing = speed; //up.dot(m_upvec)*speed;

		float t=1.0;
		if (height + rizing > m_max_height) 
			t = min( (m_max_height - height) / rizing, 1.0);

		m_lookAt += m_upvec*t*speed; //up*t*speed;
		m_lookFrom += m_upvec*t*speed; //up*t*speed;
	}
	if( (GetAsyncKeyState(CNavigation::m_down) &0x8000) )
	{
		// check height above plane through origin
		float height = m_lookFrom.dot(m_upvec);
		float rizing = speed; //up.dot(m_upvec)*speed;

		float t=1.0;
		if (height - rizing < m_min_height) 
			t = min( (height - m_min_height) / rizing, 1.0);

		m_lookAt -= m_upvec*t*speed; //up*t*speed;
		m_lookFrom -= m_upvec*t*speed; //up*t*speed;
	}

	//--- TURN ---
	if( (GetAsyncKeyState(CNavigation::m_turn_left) &0x8000) ||
		(GetAsyncKeyState(CNavigation::m_alt_turn_left) &0x8000))
	{
		// rotate lookat around upvec
		Vec3f lookVector = m_lookAt - m_lookFrom;
		Mtx3f m;
		m.rotAxis(m_upvec, turnSpeed); 
		m_lookAt = m_lookFrom + m * lookVector; // rotate lookVector around up-vector
	}
	if( (GetAsyncKeyState(CNavigation::m_turn_right) &0x8000) ||
		(GetAsyncKeyState(CNavigation::m_alt_turn_right) &0x8000))
	{
		// rotate lookat around upvec
		Vec3f lookVector = m_lookAt - m_lookFrom;
		Mtx3f m;
		m.rotAxis(m_upvec, -turnSpeed); 
		m_lookAt = m_lookFrom + m * lookVector; // rotate lookVector around up-vector
	}


	// update lookvec and screenXaxis
	lookVec = m_lookAt - m_lookFrom;
	lookVec.normalize();
	screenXaxis = lookVec.cross(m_upvec);
	screenXaxis.normalize();
	forward = lookVec - m_upvec*(lookVec.dot(m_upvec));
	forward.normalize();


	if( (GetAsyncKeyState(CNavigation::m_turn_up) &0x8000) ||
		(GetAsyncKeyState(CNavigation::m_alt_turn_up) &0x8000))
	{
		// rotate lookat around screenXaxis
		Vec3f lookVector = m_lookAt - m_lookFrom;
		// rotation is max 1.5*pi/4
		float turnAngle = turnSpeed;
		float cosangle = lookVec.dot(forward);
		float angle = acos(cosangle); // acos returns radians
		if(m_upvec.dot(lookVec) >= 0) // current angle is positive upwards
		{
			if(angle + turnAngle > m_max_up_down_rot)
				turnAngle = m_max_up_down_rot - angle;
		}
		Mtx3f m;
		m.rotAxis(screenXaxis, turnAngle); 
		m_lookAt = m_lookFrom + m * lookVector; // rotate lookVector around up-vector
	}
	if( (GetAsyncKeyState(CNavigation::m_turn_down) &0x8000) ||
		(GetAsyncKeyState(CNavigation::m_alt_turn_down) &0x8000))
	{
		// rotate lookat around screenXaxis
		Vec3f lookVector = m_lookAt - m_lookFrom;
		// rotation is max 1.5*pi/4
		float turnAngle = turnSpeed;
		float cosangle = lookVec.dot(forward);
		float angle = acos(cosangle); // acos returns radians
		if(m_upvec.dot(lookVec) <= 0) // current angle is positive downwards
		{
			if(angle + turnAngle > m_max_up_down_rot)
				turnAngle = m_max_up_down_rot - angle;
		}
		Mtx3f m;
		m.rotAxis(screenXaxis, -turnAngle); 
		m_lookAt = m_lookFrom + m * lookVector; // rotate lookVector around up-vector
	}

	// recompute upvector
	if(! m_lock_upvec)
	{
		Vec3f lookVec = m_lookAt - m_lookFrom;
		lookVec.normalize();
		Vec3f screenXaxis = lookVec.cross(m_upvec);
		screenXaxis.normalize();
		
		Vec3f up = screenXaxis.cross(lookVec);
		up.normalize();
		m_upvec = up;
	}
}
