#ifndef OBJECT_SPACEWAR_H
#define OBJECT_SPACEWAR_H

#include "l3ds/l3dsrenderer.h"
#include "l3ds/collisiondetection.h"
#include "camera.h"
#include "vecmath.h"
#include <vector>

class CNature;

class Object
{
public:
	Object();
	virtual ~Object();

	virtual void render(Camera cam); // virtual means that deriving classes can override the render()-method

	virtual void update(double currentTime) ; 

	bool isColliding(Object& obj2); // checks for collision with another object

	void setBaseTexture(int texID) { m_baseTexID = texID; }
	
	static void loadExplosionTextures();
		
	float getRadius();					 // returns the radius of the object in world space
	Vec3f getCenterPoint();				 // returns the center point of the object in world space
	BBox3f getBoundingBoxInWorldSpace(); // returns an axis aligned bounding box for this object in world space
		
	void getPositionAndOrientation(Vec3f& pos, Vec3f& lookVec, Vec3f& upVec);
	void setPositionAndOrientation(Vec3f pos,  Vec3f  lookVec, Vec3f  upVec);
		
	// misc per object data

	enum { NORMAL, DEAD, EXPLODING };
	int				m_status;
	int				m_ownerID;		// this is the playerID of the owner of this object, e.g. if this object is a missile it is the playerID of the player that launched this missile
	int				m_killerID;		// id of the last missile that hit this object. This is necessary to avoid an object being killed twice by the same missile. (UDP forces us to send the hit information several times)
//	BBox3f			m_bbox;			// bounding box of object
//	AABB*			m_AABB;			// Axis Aligned Bounding Box hierarchy
	Mtx4f			m_mtx;			// this defines the position, rotation, scaling etc
	L3DSRenderer*	m_l3dsObj;		// 3D model
	

	double			m_timeLastUpdate; // we need to know how long time has gone since the last update in order to advance movements and texture animations
	double			m_timeOfExplosion;	

	bool			m_useBackFaceCulling;

protected: // protected means that deriving classes (e.g. Missile) can access these members
	int				m_baseTexID;	  // base texture for the object
	int				m_currentTexID;	  // identifies the currently used texture, i.e., base or explosion textures
	static  std::vector<int> m_explTexIDs; // explosion texture IDs
	
};
#endif // OBJECT_SPACEWAR_H