#pragma warning (disable:4786)

#include <windows.h>
#include "player.h"
#include <GL/GLee.h>
#include <gl/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include "tex/texturemanager.h"

#ifndef M_PI
#define M_PI       3.14159265358979323846
#endif

// Storage for some preloaded models
L3DSRenderer Player::m_fighter;		

Player::Player() : m_score(0), m_health(1)
{
	// constructor. Do allocations per object here

	static bool bFirstTime=true; // static means that this object is the very same for all invocations of this method
	if(bFirstTime)
	{
		bFirstTime=false;
		createModels();
	}
}

Player::~Player()
{
	// destructor. Do deallocations per object here
}

void Player::createModels()
{
	// Load models and create bounding box hierarchies
	// Load some models into memory only first time this method is called
		
	m_fighter.Load("media/models/", "SpaceFighter01.3ds"); // Loads 3ds-model
	
	Mtx4f m0, m1, m2, m3;
	m0.loadIdentity();
	m1.loadIdentity();
	m2.loadIdentity();
	m3.loadIdentity();
	m1.scale(10, 10, 10);
	m2.rotX(-M_PI*0.5f);
	m3.rotY(-M_PI*1.0f);
	Vec3f center = m_fighter.GetBBox().centerPoint();
	m0.translate(-center[0], -center[1], -center[2]);
	m_fighter.getMatrix() = m3*m2*m1*m0; // translate to center, scale by 0.01 and rotate -90 degrees around x-axis
}

// Creating player0, i.e., ourself
void Player::initPlayer0()
{
	Vec3f pos;
	pos[0] = -100.0f + 200*(float)rand()/(float)RAND_MAX ; // random value between -100 and 100
	pos[1] = 10.0f;
	pos[2] = -100.0f + 200*(float)rand()/(float)RAND_MAX ; // random value between -100 and 100
	
	// face center of the world
	Vec3f lookVec = Vec3f(0,0,0)-pos; 
	lookVec.normalize();

	Vec3f upVec(0,1,0);
	setPositionAndOrientation(pos, lookVec, upVec);
	
	m_l3dsObj = &m_fighter;
}

void Player::initRemotePlayer()
{
	// set the same model and AABB hierarchy
	m_l3dsObj = &m_fighter;
}

