#include <stdio.h>
#include "winsock2.h"
#include "sockets.h"

CMiniSockets::CMiniSockets()
{
	m_RecvAddr = new struct sockaddr_in; // this is to avoid having to include winsock2.h in sockets.h
}

CMiniSockets::~CMiniSockets()
{
	//---------------------------------------------
	// When the application is finished sending, close the socket.
	printf("Finished sending. Closing socket.\n");
	closesocket(m_socketID);
	
	WSACleanup();
	//---------------------------------------------
}

void CMiniSockets::createSendSocket(int Port) // Creates a sending socket
{
	WSADATA wsaData;
	SOCKET SendSocket;
	char* SendBuf = m_buffer;	// Care must be taken if this is larger than 1500 bytes - the standard message size. Overflow will cause sending of several packages. Receiving socket treats it as several messages.
	int BufLen = 1024;

	m_port = Port;
	
	//---------------------------------------------
	// Initialize Winsock
	WSAStartup(MAKEWORD(2,2), &wsaData);

	//---------------------------------------------
	// Create a socket for sending data
	SendSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	m_socketID = SendSocket;

	void test();
//	test();

	//---------------------------------------------
	// Set up the RecvAddr structure with the IP address of
	// the receiver (in this example case "123.456.789.1")
	// and the specified port number.
	m_RecvAddr->sin_family = AF_INET;
	m_RecvAddr->sin_port = htons(Port);
	//m_RecvAddr->sin_addr.s_addr = inet_addr("127.0.0.1");		// send to own computer
	//RecvAddr.sin_addr.s_addr = inet_addr("255.255.255.255");	// limited broadcast (i.e., only to LAN)
	m_RecvAddr->sin_addr.s_addr = INADDR_BROADCAST;			// limited broadcast (i.e., only to LAN)
	//m_RecvAddr->sin_addr.s_addr = inet_addr("192.168.1.101");
	//m_RecvAddr->sin_addr.s_addr = inet_addr("129.16.20.220");

	bind(SendSocket, (SOCKADDR *) m_RecvAddr, sizeof(*m_RecvAddr));
	
	//---------------------------------------------
	// Enable broadcast capability
	BOOL bOptVal = TRUE;
	int bOptLen = sizeof(BOOL);
	int iOptVal;
	int iOptLen = sizeof(int);

	if (setsockopt(SendSocket, SOL_SOCKET, SO_BROADCAST, (char*)&bOptVal, bOptLen) != SOCKET_ERROR) {
		printf("Set SO_BROADCAST: ON\n");
	}

	if (getsockopt(SendSocket, SOL_SOCKET, SO_BROADCAST, (char*)&iOptVal, &iOptLen) != SOCKET_ERROR) {
		printf("SO_BROADCAST Value: %ld\n", iOptVal);
	}
	//---------------------------------------------
}

void CMiniSockets::sendMessage(const char* str, int len) // str should be null-terminated string of max 1024 bytes
{
	// Send a datagram to the receiver
	sendto(m_socketID, str, len, 0, (SOCKADDR *) m_RecvAddr, sizeof(*m_RecvAddr));

	return;
} 

void CMiniSockets::createRecvSocket(int Port) // Creates a receiving socket
{
  WSADATA wsaData;
  SOCKET RecvSocket;

  m_port = Port;

  //-----------------------------------------------
  // Initialize Winsock
  WSAStartup(MAKEWORD(2,2), &wsaData);

  //-----------------------------------------------
  // Create a receiver socket to receive datagrams
  RecvSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  m_socketID = RecvSocket;

  //-----------------------------------------------
  // Bind the socket to any address and the specified port.
  m_RecvAddr->sin_family = AF_INET;
  m_RecvAddr->sin_port = htons(Port);
  m_RecvAddr->sin_addr.s_addr = htonl(INADDR_ANY);

  bind(RecvSocket, (SOCKADDR *) m_RecvAddr, sizeof(*m_RecvAddr));

  //---------------------------------------------
  // Enable broadcast capability
  BOOL bOptVal = TRUE;
  int bOptLen = sizeof(BOOL);
  int iOptVal;
  int iOptLen = sizeof(int);

  if (getsockopt(RecvSocket, SOL_SOCKET, SO_BROADCAST, (char*)&iOptVal, &iOptLen) != SOCKET_ERROR) {
	  printf("SO_BROADCAST Value: %ld\n", iOptVal);
  }
  if (setsockopt(RecvSocket, SOL_SOCKET, SO_BROADCAST, (char*)&bOptVal, bOptLen) != SOCKET_ERROR) {
	  printf("Set SO_BROADCAST: ON\n");
  }
  if (getsockopt(RecvSocket, SOL_SOCKET, SO_BROADCAST, (char*)&iOptVal, &iOptLen) != SOCKET_ERROR) {
	  printf("SO_BROADCAST Value: %ld\n", iOptVal);
  }

  //-----------------------------------------------
  // Set the socket I/O mode: In this case FIONBIO
  // enables or disables the blocking mode for the 
  // socket based on the numerical value of iMode.
  // If iMode = 0, blocking is enabled; 
  // If iMode != 0, non-blocking mode is enabled.
  u_long iMode = 1;
  ioctlsocket(RecvSocket, FIONBIO, &iMode);
}

const char* CMiniSockets::receiveMessage()	   //  
{
  //-----------------------------------------------
  // Call the recvfrom function to receive datagrams
  // on the bound socket.
  sockaddr_in SenderAddr;
  int SenderAddrSize = sizeof(SenderAddr);

  m_buffer[0]='\0';
  recvfrom(m_socketID, m_buffer, 1024, 0, (SOCKADDR *)&SenderAddr, &SenderAddrSize);
//  while (recvfrom(m_socketID, m_buffer, 1024, 0, (SOCKADDR *)&SenderAddr, &SenderAddrSize)
//	  != WSAEWOULDBLOCK) {
//	  // loop until no more messages exist. I.e, get latest message.
//  }

  return m_buffer;
}


void test()
{
	//----------------------
	// Declare and initialize variables
	hostent* remoteHost;
	char* host_name;
	unsigned int addr;
	
	//----------------------
	// User inputs name of host
	printf("Input name of host: ");
	host_name = (char*) malloc(sizeof(char*)*16);
	fgets(host_name, 16, stdin);
	
	// If the user input is an alpha name for the host, use gethostbyname()
	// If not, get host by addr (assume IPv4)
	if (isalpha(host_name[0])) {   /* host address is a name */
		host_name[strlen(host_name)-1] = '\0'; /* NULL TERMINATED */
		remoteHost = gethostbyname(host_name);
	}
	else  { 
		addr = inet_addr(host_name);
		remoteHost = gethostbyaddr((char *) &addr, 4, AF_INET);
	}
	
	if (WSAGetLastError() != 0) {
		if (WSAGetLastError() == 11001)
			printf("Host not found...\nExiting.\n");
	}
	else
		printf("error#:%ld\n", WSAGetLastError());
	
	//Sleep(750);
	
	// The remoteHost structure can now be used to
	// access information about the host
	
}
