// INCLUDES ///////////////////////////////////////////////

//#define WIN32_LEAN_AND_MEAN  

// has the GUID library been included?
#ifndef INITGUID
#define INITGUID
#endif

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <direct.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <io.h>
#include <fcntl.h>
#include <objbase.h>
#include <fstream.h>

// DirectX Includes
#include <dsound.h>		//DirectSound
#include <dmksctrl.h>	//DirectMusic
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>

#include "DXLib.h"

// Macros
// initializes a direct draw struct
#define DDraw_InitStruct(ddstruct) { memset(&ddstruct,0,sizeof(ddstruct)); ddstruct.dwSize=sizeof(ddstruct); }
#define MULTI_TO_WIDE( x,y )  MultiByteToWideChar( CP_ACP,MB_PRECOMPOSED, y,-1,x,_MAX_PATH);

// DEFINES ////////////////////////////////////////////////

// TYPES //////////////////////////////////////////////////

// PROTOTYPES /////////////////////////////////////////////

// EXTERNALS /////////////////////////////////////////////

const HWND main_window_handle=NULL; // save the window handle
extern HINSTANCE main_instance; // save the instance

// DirectSound
LPDIRECTSOUND lpds = NULL;	// the directsound object
DSBUFFERDESC dsbd;	// ds buffer desc
WAVEFORMATEX pcmfx;	// holds format desc

// DirectMusic
IDirectMusicPerformance	*dmp = NULL;	// DirectMusic Performance
IDirectMusicLoader *dml = NULL;			// DirectMusic Loader

// FUNCTIONS //////////////////////////////////////////////
// sin and cos tables


// DirectMusic
int CMusic::LoadMIDI (char *filename)
{
DMUS_OBJECTDESC ObjDesc; 
HRESULT hr;
IDirectMusicSegment* pSegment = NULL;

// get current working directory
char szDir[_MAX_PATH];
WCHAR wszDir[_MAX_PATH]; 

if(_getcwd( szDir, _MAX_PATH ) == NULL)
  {
  return 0;
  } // end if

MULTI_TO_WIDE(wszDir, szDir);

// tell the loader were to look for files
hr = dml->SetSearchDirectory(GUID_DirectMusicAllTypes,wszDir, FALSE);

if (FAILED(hr)) 
   {
   return 0;
   } // end if

// convert filename to wide string
WCHAR wfilename[_MAX_PATH]; 
MULTI_TO_WIDE(wfilename, filename);
 
// setup object description
DDraw_InitStruct(ObjDesc);
ObjDesc.guidClass = CLSID_DirectMusicSegment;
wcscpy(ObjDesc.wszFileName, wfilename );
ObjDesc.dwValidData = DMUS_OBJ_CLASS | DMUS_OBJ_FILENAME;
 
// load the object and query it for the IDirectMusicSegment interface
// This is done in a single call to IDirectMusicLoader::GetObject
// note that loading the object also initializes the tracks and does 
// everything else necessary to get the MIDI data ready for playback.

hr = dml->GetObject(&ObjDesc,IID_IDirectMusicSegment, (void**) &pSegment);

if (FAILED(hr))
   return 0;
 
// ensure that the segment plays as a standard MIDI file
// you now need to set a parameter on the band track
// Use the IDirectMusicSegment::SetParam method and let 
// DirectMusic find the trackby passing -1 (or 0xFFFFFFFF) in the dwGroupBits method parameter.

hr = pSegment->SetParam(GUID_StandardMIDIFile,-1, 0, 0, (void*)dmp);

if (FAILED(hr))
   return 0;
  
// This step is necessary because DirectMusic handles program changes and 
// bank selects differently for standard MIDI files than it does for MIDI 
// content authored specifically for DirectMusic. 
// The GUID_StandardMIDIFile parameter must be set before the instruments are downloaded. 

// The next step is to download the instruments. 
// This is necessary even for playing a simple MIDI file 
// because the default software synthesizer needs the DLS data 
// for the General MIDI instrument set
// If you skip this step, the MIDI file will play silently.
// Again, you call SetParam on the segment, this time specifying the GUID_Download parameter:

hr = pSegment->SetParam(GUID_Download, -1, 0, 0, (void*)dmp);

if (FAILED(hr))
   return 0;

// at this point we have MIDI loaded and a valid object

dms  = pSegment;
dmss = NULL;
state       = 1;
 
// return id
return 1;
}

int CMusic::Play ( void )
{
	if (FAILED(dmp->PlaySegment (dms, 0, 0, &dmss)))
	{
		MessageBox (main_window_handle,"Error","Unable to play music segment",0);
		return 1;
	}
	return 1;
}

int CMusic::Stop ( void )
{
	if (FAILED(dmp->Stop (dms,NULL,0,0)))
	{
		MessageBox (main_window_handle,"Error","Unable to stop segment",0);
		return 0;
	}
	return 1;
}

bool CMusic::isPlaying ( void )
{
	if (dmp->IsPlaying (dms,NULL) == S_OK)
	{
		return true;
	}
	else
	{
		return false;
	}
}

int CMusic::Release ( void )
{
	dms->SetParam (GUID_Unload, -1, 0, 0, (void*)dmp);
	dms->Release ();
	dms = NULL;
	return 1;
}

int DMusic_Setup ( void )
{
	if (FAILED(CoInitialize (NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to setup COM for DM",0);
		return 0;
	}

	if (FAILED(CoCreateInstance (CLSID_DirectMusicPerformance, NULL, CLSCTX_INPROC, IID_IDirectMusicPerformance, (void**)&dmp)))
	{
		MessageBox (main_window_handle,"Error","Unable to create DMPerformance COM object",0);
		return 0;
	}

	if (lpds)
	{
	if (FAILED(dmp->Init (NULL,lpds,main_window_handle)))
	{
		MessageBox (main_window_handle,"Error","Unable to Initialize DM",0);
		return 0;
	}
	}
	else
	{
	if (FAILED(dmp->Init (NULL,NULL,main_window_handle)))
	{
		MessageBox (main_window_handle,"Error","Unable to Initialize DM",0);
		return 0;
	}
	}

	if (FAILED(dmp->AddPort (NULL)))
	{
		MessageBox (main_window_handle,"Error","Unable to add port to performance",0);
		return 0;
	}

	if (FAILED(CoCreateInstance (CLSID_DirectMusicLoader, NULL, CLSCTX_INPROC,IID_IDirectMusicLoader, (void**)&dml)))
	{
		MessageBox (main_window_handle,"Error","Unable to create DM Loader",0);
		return 0;
	}

	// Finito
	return 1;
}

int DMusic_Release ( void )
{
	if (dmp)
	{
		dmp->Stop (NULL,NULL,0,0);
		dmp->CloseDown ();
		dmp->Release ();
	}
	if (dml)
		dml->Release ();

	CoUninitialize ();
	return 1;
}

// DirectSound
int CSound::Setup (int size, int flags)
{
	this->size = size;
	this->rate = rate;

	DDraw_InitStruct (dsbd);
	memset (&pcmfx,0,sizeof(pcmfx));
	pcmfx.wFormatTag = WAVE_FORMAT_PCM;
	pcmfx.nChannels = 1;			// mono
	pcmfx.nSamplesPerSec = 11025;	// 11kHz/22kHz/etc
	pcmfx.nBlockAlign = 1;
	pcmfx.nAvgBytesPerSec = pcmfx.nSamplesPerSec * pcmfx.nBlockAlign;
	pcmfx.wBitsPerSample = 8;		// 8bit
	pcmfx.cbSize = 0;

	dsbd.dwFlags = flags;
	dsbd.dwBufferBytes = size;	// # of sec  *  rate
	dsbd.lpwfxFormat = &pcmfx;

	if (FAILED(lpds->CreateSoundBuffer (&dsbd,&buffer,NULL)))
	{
		MessageBox (main_window_handle,"Error","Cannot create sound buffer",0);
	}

	state = 1;	// ready
	return 1;
}

int CSound::Pause ( void )
{
	if (state > 1)
		buffer->Stop ();
	state = 1;
	return 1;
}

int CSound::Release ()
{
	state = 0;
	buffer->Release ();
	buffer = NULL;
	return 1;
}

int CSound::Play (bool loop)
{
	int flags = 0;
	if (loop)
		flags = DSBPLAY_LOOPING;

	if (FAILED(buffer->Play (0,0,flags)))
	{
		MessageBox (main_window_handle,"Error","Unable to Play sound!",0);
		return 0;
	}

	state = 2;	// Playing
	if (loop)
		state = 3;	// Looping
	return 1;
}

int CSound::Stop ( void )
{
	if (state > 1)
	{
		buffer->Stop ();
		buffer->SetCurrentPosition (0);
		state = 1;	// ready to rumble
	}
	return 1;
}

int CSound::SetVolume ( int volume )
{
	if (FAILED(buffer->SetVolume ((DWORD)(-30*(100 - volume)))))
	{
		MessageBox (main_window_handle,"Error","Unable to set volume",0);
		return 0;
	}
	return 1;
}

int CSound::SetFrequency ( int freq )
{
	if (FAILED(buffer->SetFrequency (freq)))
	{
		MessageBox (main_window_handle,"Error","Cannot set frequency",0);
		return 0;
	}
	return 1;
}

int CSound::SetPan ( int pan )
{
	if (FAILED(buffer->SetPan (pan)))
	{
		MessageBox (main_window_handle,"Error","Unable to pan sound",0);
		return 0;
	}
	return 1;
}

int CSound::LoadSound ( char *name )
{
// this function loads a .wav file, sets up the directsound 
// buffer and loads the data into memory, the function returns 
// the id number of the sound


HMMIO 			hwav;    // handle to wave file
MMCKINFO		parent,  // parent chunk
                child;   // child chunk
WAVEFORMATEX    wfmtx;   // wave format structure

UCHAR *snd_buffer,       // temporary sound buffer to hold voc data
      *audio_ptr_1=NULL, // data ptr to first write buffer 
	  *audio_ptr_2=NULL; // data ptr to second write buffer

DWORD audio_length_1=0,  // length of first write buffer
	  audio_length_2=0;  // length of second write buffer
			
// set up chunk info structure
parent.ckid 	    = (FOURCC)0;
parent.cksize 	    = 0;
parent.fccType	    = (FOURCC)0;
parent.dwDataOffset = 0;
parent.dwFlags		= 0;

// copy data
child = parent;

// open the WAV file
if ((hwav = mmioOpen(name, NULL, MMIO_READ | MMIO_ALLOCBUF))==NULL)
    return 0;

// descend into the RIFF 
parent.fccType = mmioFOURCC('W', 'A', 'V', 'E');

if (mmioDescend(hwav, &parent, NULL, MMIO_FINDRIFF))
    {
    // close the file
    mmioClose(hwav, 0);

    // return error, no wave section
    return 0; 	
    } // end if

// descend to the WAVEfmt 
child.ckid = mmioFOURCC('f', 'm', 't', ' ');

if (mmioDescend(hwav, &child, &parent, 0))
    {
    // close the file
    mmioClose(hwav, 0);

    // return error, no format section
    return 0; 	
    } // end if

// now read the wave format information from file
if (mmioRead(hwav, (char *)&wfmtx, sizeof(wfmtx)) != sizeof(wfmtx))
    {
    // close file
    mmioClose(hwav, 0);

    // return error, no wave format data
    return 0;
    } // end if

// make sure that the data format is PCM
if (wfmtx.wFormatTag != WAVE_FORMAT_PCM)
    {
    // close the file
    mmioClose(hwav, 0);

    // return error, not the right data format
    return 0; 
    } // end if

// now ascend up one level, so we can access data chunk
if (mmioAscend(hwav, &child, 0))
   {
   // close file
   mmioClose(hwav, 0);

   // return error, couldn't ascend
   return 0; 	
   } // end if

// descend to the data chunk 
child.ckid = mmioFOURCC('d', 'a', 't', 'a');

if (mmioDescend(hwav, &child, &parent, MMIO_FINDCHUNK))
    {
    // close file
    mmioClose(hwav, 0);

    // return error, no data
    return 0; 	
    } // end if

// finally!!!! now all we have to do is read the data in and
// set up the directsound buffer

// allocate the memory to load sound data
snd_buffer = (UCHAR *)malloc(child.cksize);

// read the wave data 
mmioRead(hwav, (char *)snd_buffer, child.cksize);

// close the file
mmioClose(hwav, 0);

// set rate and size in data structure
rate  = wfmtx.nSamplesPerSec;
size  = child.cksize;
state = 1;

// copy data into sound buffer
if (FAILED(buffer->Lock(0, child.cksize, (void **) &audio_ptr_1, &audio_length_1, (void **)&audio_ptr_2, &audio_length_2, DSBLOCK_FROMWRITECURSOR)))
	 return(0);

// copy first section of circular buffer
memcpy(audio_ptr_1, snd_buffer, audio_length_1);

// copy last section of circular buffer
memcpy(audio_ptr_2, (snd_buffer+audio_length_1),audio_length_2);

// unlock the buffer
if (FAILED(buffer->Unlock(audio_ptr_1, 
									    audio_length_1, 
									    audio_ptr_2, 
									    audio_length_2)))
 							     return(0);

// release the temp buffer
free(snd_buffer);

// return id
return 1;
}

int DSound_Setup ( void )
{
	if (FAILED(DirectSoundCreate (NULL,&lpds,NULL)))
	{
		return 0;
	}
	if (FAILED(lpds->SetCooperativeLevel (main_window_handle,DSSCL_NORMAL)))
	{
		return 0;
	}
	// Ready to rock and roll
	return 1;
}

int DSound_Release ( void )
{
	if (lpds)
		lpds->Release ();
	return 1;
}

// DIRECT DRAW
