// 
// http://www.un4seen.com/bass.html

#include <windows.h>
#include <stdlib.h>
#include "bass.h"
#include "bassound.h"
#include "stdio.h"


/* Display error messages */
void Error(const char *es)
{
	char mes[200];
	sprintf(mes,"%s\n(error code: %d)",es,BASS_ErrorGetCode());
	MessageBox(NULL,mes,0,0);
}


float CBassSound::getCPUUsage()
{ 
	return BASS_GetCPU();
}
			
void CBassSound::loadMP3(const char* file)
{
	m_type = STREAM;
	// Streamable files: wav/aif/mp3/mp2/mp1/ogg
	if (m_handle=BASS_StreamCreateFile(FALSE,file,0,0,0)) 
	{
		;
	} else
		Error("Can't open stream");
}

void CBassSound::unload()
{
	if (m_type == STREAM) {
		BASS_StreamFree(m_handle);
	}
	else if (m_type == SAMPLE) {
		/* Free the sample from memory */
		BASS_SampleFree(m_handle);
	}
}
				
void CBassSound::play()
{

	if (m_type == STREAM) {
		/* Play the stream (continue from current position) */
		if (!BASS_ChannelPlay(m_handle,FALSE)) Error("Can't play stream");
	}
	else if (m_type == SAMPLE) {
		// Play the sample at default rate, volume=50, random pan position
		HCHANNEL ch=BASS_SampleGetChannel(m_handle,FALSE);
		BASS_ChannelSetAttributes(ch,-1,50,0); 
		if (!BASS_ChannelPlay(ch,FALSE)) Error("Can't play sample");
	}
}

void CBassSound::setAttributes(int vol, int pan, int freq)
{
	HCHANNEL ch = (m_type == SAMPLE) ? BASS_SampleGetChannel(m_handle,FALSE) : m_handle;

	// handle	The channel handle... a HCHANNEL, HMUSIC, HSTREAM. 
	// freq		The sample rate... 100 (min) - 100000 (max), 0 = original rate (when the channel was created), -1 = leave current. 
	// volume	The volume... 0 (silent) - 100 or what's been set with the BASS_CONFIG_MAXVOL config option (max), -1 = leave current. 
	// pan		The panning position... -100 (left) - 100 (right), -101 = leave current. 
	BASS_ChannelSetAttributes(ch, freq, vol, pan); 
		
}

void CBassSound::pause()
{
	/* Stop the stream */
	BASS_ChannelStop(m_handle);
}
void CBassSound::restart()
{
	/* Play the stream from the start */
	BASS_ChannelPlay(m_handle,TRUE);
}


void CBassSound::loadWav(const char* file)
{
	m_type = SAMPLE;
	// Sample files: wav/aif
	/* Load a sample from "file" and give it a max of 5 simultaneous
	playings using playback position as override decider */
	if (m_handle=BASS_SampleLoad(FALSE,file,0,0,5,BASS_SAMPLE_OVER_POS)) {
	} else
		Error("Can't load sample");
}

void CBassSound::pauseAllSounds()
{
	/* Pause output */
	BASS_Pause();
}

void CBassSound::resumeAllSounds()
{					/* Resume output */
	BASS_Start();
}


CBassSound::CBassSound()
{
	static bool bFirstTime = true;
	if (bFirstTime)
	{
		bFirstTime = false;

		// check the correct BASS was loaded
		if (HIWORD(BASS_GetVersion())!=BASSVERSION) {
			MessageBox(0,"An incorrect version of BASS.DLL was loaded",0,MB_ICONERROR);
		}

		if (!BASS_Init(-1,44100,0,NULL,NULL))
			Error("Can't initialize device");
		
	}
}

CBassSound::~CBassSound()
{

	/* Close output */
	// BASS_Free(); // this should be done when closing application
}
