#define INITGUID
#include <dmusici.h>

// The tutorial uses three interface pointers, which are declared as follows:

IDirectMusicLoader8*  g_pLoader           = NULL;
IDirectMusicPerformance8* g_pPerformance  = NULL;
IDirectMusicSegment8*   g_pSegment        = NULL;

// All the code in this simple application is included in the WinMain function. 
// The application has no main window, so it can proceed straight to the creation of
// COM and two objects: the loader and the performance:.

bool PlayASound()
{
	CoInitialize(NULL);
	
	CoCreateInstance(CLSID_DirectMusicLoader, NULL, 
		CLSCTX_INPROC, IID_IDirectMusicLoader8,
		(void**)&g_pLoader);
	
	CoCreateInstance(CLSID_DirectMusicPerformance, NULL,
		CLSCTX_INPROC, IID_IDirectMusicPerformance8,
		(void**)&g_pPerformance );


	g_pPerformance->InitAudio( 
		NULL,              // IDirectMusic interface not needed.
		NULL,              // IDirectSound interface not needed.
		NULL,              // Window handle.
		DMUS_APATH_SHARED_STEREOPLUSREVERB,  // Default audiopath type.
		64,                // Number of performance channels.
		DMUS_AUDIOF_ALL,   // Features on synthesizer.
		NULL               // Audio parameters; use defaults.
		);
	

	// Find the Windows media directory.
	
	CHAR strPath[512];
	if( GetWindowsDirectory( strPath, MAX_PATH+1 ) == 0 )
        return 0;
	strcat( strPath, "\\media" );
	
	// Convert to Unicode.
	
	WCHAR wstrSearchPath[MAX_PATH + 1];
	MultiByteToWideChar( CP_ACP, 0, strPath, -1, 
		wstrSearchPath, MAX_PATH );
	wstrSearchPath[MAX_PATH] = 0;
	
	// Set the search directory.
	
	g_pLoader->SetSearchDirectory( 
		GUID_DirectMusicAllTypes, // Types of files sought.
		wstrSearchPath,           // Where to look.
		FALSE                     // Don't clear object data.
		);
	
//	In the call to SetSearchDirectory, the fClear parameter is set to FALSE because 
//	there is no danger of accidentally reloading objects from the wrong directory. 
//	This is likely to happen only if the application is loading identically named 
//	objects from different folders.
//	Now that the loader knows where to look for the file, it can load it as a segment:
	
	WCHAR wstrFileName[MAX_PATH] = L"ding.wav";
	
	if (FAILED(g_pLoader->LoadObjectFromFile(
		CLSID_DirectMusicSegment, // Class identifier.
		IID_IDirectMusicSegment8, // ID of desired interface.
		wstrFileName,             // Filename.
		(LPVOID*) &g_pSegment     // Pointer that receives interface.
		)))
	{
		MessageBox( NULL, "Media not found, sample will now quit.", 
			"DirectMusic Tutorial", MB_OK );
		g_pPerformance->CloseDown();
		g_pLoader->Release(); 
		g_pPerformance->Release();
		CoUninitialize();
		return 0;
	}
	
	
	g_pSegment->Download( g_pPerformance );
	
//	To play the file, pass the segment interface to IDirectMusicPerformance8::PlaySegmentEx. 
//	This method offers many options for playback, but to play a segment immediately on the 
//	default audiopath, all the parameters except the first can be NULL or 0:
	
	g_pPerformance->PlaySegmentEx(
		g_pSegment,  // Segment to play.
		NULL,        // Not used.
		NULL,        // For transitions. 
		0,           // Flags.
		0,           // Start time; 0 is immediate.
		NULL,        // Pointer that receives segment state.
		NULL,        // Object to stop.
		NULL         // Audiopath, if not default.
		);  
	MessageBox( NULL, "Click OK to Exit.", "Play Audio", MB_OK );
	
//	To exit an audio application cleanly, you must perform four main steps: 
//	
//	1. Stop any playing segments by calling IDirectMusicPerformance8::Stop. 
//	2. Close down the performance. The IDirectMusicPerformance8::CloseDown method 
//	   performs miscellaneous cleanup tasks and releases internal references to objects. 
//	3. Release all interfaces. 
//	4. Close COM. 
//	The following code from the WinMain function in the tutorial sample is called 
//	when the dialog box is closed.
		
		g_pPerformance->Stop(
		NULL, // Stop all segments.
		NULL, // Stop all segment states.
		0,    // Do it immediately.
		0     // Flags.
		);
	
	g_pPerformance->CloseDown();
	
	g_pLoader->Release(); 
	g_pPerformance->Release();
	g_pSegment->Release();
	
	CoUninitialize();
    
	return 0;  // Return value for WinMain.
}      // End of WinMain.
	
