#pragma warning (disable:4786)
// includes
#include "windows.h"
#include <GL/GLee.h>
#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include "TGAImage.hpp"
#include "texturemanager.h"
#include "ppmimage.h"

#include <string>
#include <fstream>


std::map<std::string, GLuint> TextureManager::m_textures;

GLuint TextureManager::load(const std::string &filename)
{
	if (filename.empty()) return 0;
	
	GLuint texture;
	
	std::map<std::string, GLuint>::iterator it = m_textures.find(filename);
	
	if (it != m_textures.end())
		texture = it->second;
	else
	{
		glGenTextures(1, &texture);
		std::string fn = filename;
		unsigned int pos = filename.find(".tga");
		if( pos >= filename.length())
		{
			int width, height;
			unsigned char* pixels=0;
			std::string ppm(".ppm");
			fn.replace(fn.length()-4, 4, ppm);
			if( ! fConvertToPPM(filename.c_str()) || ! fReadPPM(fn.c_str(), width, height, pixels) )
			{
				printf("could not open texture file %s.\n", filename.begin());
				return 0;
			}
			
			glBindTexture(GL_TEXTURE_2D, texture);
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, pixels);
			// we should deallocate pixels now
		}
		else
		{
			char tmp[255]; strcpy(tmp, fn.c_str());
			if(! TextureManager::assertTGA(tmp) )
			{
				printf("could not open texture file %s.", filename.c_str());
				return 0;
			}
			
			bleifrei::img::TGAImage image(std::ifstream(tmp, std::ios::binary));
			glBindTexture(GL_TEXTURE_2D, texture);
			{
				switch (image.getFormat()) {
				case bleifrei::img::ImageFormatL8 :
					gluBuild2DMipmaps(GL_TEXTURE_2D, 1, image.getWidth(), image.getHeight(), GL_LUMINANCE, GL_UNSIGNED_BYTE, image.getData());
					break;
				case bleifrei::img::ImageFormatA8 :
					gluBuild2DMipmaps(GL_TEXTURE_2D, 1, image.getWidth(), image.getHeight(), GL_ALPHA, GL_UNSIGNED_BYTE, image.getData());
					break;
					//case 16 :
					//glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE16, image.getWidth(), image.getHeight(), 0, GL_LUMINANCE, GL_UNSIGNED_SHORT, image.getData());
					//gluBuild2DMipmaps(GL_TEXTURE_2D, 1, image.getWidth(), image.getHeight(), GL_LUMINANCE16, GL_UNSIGNED_SHORT, image.getData());
					//break;
				case bleifrei::img::ImageFormatR8G8B8 :
					gluBuild2DMipmaps(GL_TEXTURE_2D, 3, image.getWidth(), image.getHeight(), GL_RGB, GL_UNSIGNED_BYTE, image.getData());
					break;
				case bleifrei::img::ImageFormatB8G8R8 :
					gluBuild2DMipmaps(GL_TEXTURE_2D, 3, image.getWidth(), image.getHeight(), GL_BGR_EXT, GL_UNSIGNED_BYTE, image.getData());
					break;
				case bleifrei::img::ImageFormatR8G8B8A8 :
					gluBuild2DMipmaps(GL_TEXTURE_2D, 4, image.getWidth(), image.getHeight(), GL_RGBA, GL_UNSIGNED_BYTE, image.getData());
					break;
				case bleifrei::img::ImageFormatB8G8R8A8 :
					gluBuild2DMipmaps(GL_TEXTURE_2D, 4, image.getWidth(), image.getHeight(), GL_BGRA_EXT, GL_UNSIGNED_BYTE, image.getData());
					break;
				default:
					printf("Unsupported tga format\n");
				}
			}
		}
		m_textures.insert(std::map<std::string, GLuint>::value_type(filename, texture));
	}
	
	return texture;
}

// method to create alfa texture. Alpha will be set to 0 for each rgb-texel matching value transp[3]
GLuint TextureManager::load(const std::string &filename, unsigned char transp[3], bool bSetAlphaToIntensity)
{
	if (filename.empty()) return 0;
	
	GLuint texture;
	
	std::map<std::string, GLuint>::iterator it = m_textures.find(filename);
	
	if (it != m_textures.end())
		texture = it->second;
	else
	{
		glGenTextures(1, &texture);
		std::string fn = filename;

		unsigned int pos = filename.find(".tga");
		if( pos >= filename.length())
		{
			int width, height;
			unsigned char* pixels=0;
			std::string ppm(".ppm");
			fn.replace(fn.length()-4, 4, ppm);
			if( ! fConvertToPPM(filename.c_str()) || ! fReadPPM(fn.c_str(), width, height, pixels) )
			{
				printf("could not open texture file %s.\n", filename.begin());
				return 0;
			}

			unsigned char* texels = new unsigned char[width * height * 4];
			for(int i=0; i<height; i++)
			{
				for(int j=0; j<width; j++)
				{
					unsigned char r,g,b,a;
					r = pixels[(i*width + j)*3 + 0];
					g = pixels[(i*width + j)*3 + 1];
					b = pixels[(i*width + j)*3 + 2];
					a = 255;
					
					texels[(i*width + j)*4 + 0] = r;
					texels[(i*width + j)*4 + 1] = g;
					texels[(i*width + j)*4 + 2] = b;
					
					if (bSetAlphaToIntensity) 
					{
						int _a = (r+g+b) / 3;
						a = _a;
					}
					else
					{
						if(r==transp[0] && g==transp[1] && b==transp[2])
							a = 0;
					}
					
					texels[(i*width + j)*4 + 3] = a;
				}
			}
			
			delete pixels;
			glBindTexture(GL_TEXTURE_2D, texture);
			gluBuild2DMipmaps(GL_TEXTURE_2D, 4, width, height, GL_RGBA, GL_UNSIGNED_BYTE, texels);
			// we should deallocate pixels now
		}
		else
		{
			char tmp[255]; strcpy(tmp, fn.c_str());
			if(! TextureManager::assertTGA(tmp) )
			{
				printf("could not open texture file %s.", filename.c_str());
				return 0;
			}
			
			bleifrei::img::TGAImage image(std::ifstream(tmp, std::ios::binary));
			
			// convert to RGBA
			unsigned char* texels = new unsigned char[image.getWidth() * image.getHeight() * 4];
			for(int i=0; i<image.getHeight(); i++)
			{
				for(int j=0; j<image.getWidth(); j++)
				{
					unsigned char r,g,b,a=255;
					image.getPixel( j, i, r,g,b,a);
					
					texels[(i*image.getWidth() + j)*4 + 0] = r;
					texels[(i*image.getWidth() + j)*4 + 1] = g;
					texels[(i*image.getWidth() + j)*4 + 2] = b;
					
					if(r==transp[0] && g==transp[1] && b==transp[2])
						a = 0;
					
					texels[(i*image.getWidth() + j)*4 + 3] = a;
				}
			}
			
			glBindTexture(GL_TEXTURE_2D, texture);
			gluBuild2DMipmaps(GL_TEXTURE_2D, 4, image.getWidth(), image.getHeight(), GL_RGBA, GL_UNSIGNED_BYTE, texels);
		}

		m_textures.insert(std::map<std::string, GLuint>::value_type(filename, texture));

		// we should definitely delete allocated memory for texels. 
		// However, OpenGL has a tendency to crash if textures are deallocated while still active.
		// One solution is to store pointer in a static list for texturemanager. 
	}
	
	return texture;
}


// tries to convert file to tga (leaving original file intact), if not already in tga-format
// Note: Possibly pollutes by leaving the possibly created tga-file on file system
bool TextureManager::assertTGA(char *filename)
{
	// call the image converter 'convert.exe ppmUnicodeFileName ppmTmpFileName'	
	TCHAR args[2*MAX_PATH + 100];
	args[0] = NULL;
	TCHAR FileName0[2*MAX_PATH + 100];
	TCHAR FileName1[2*MAX_PATH + 100];
	int l=(int)strlen(filename);
	
	// assert the original texture file exists
	FILE *f;
	f=fopen(filename,"rb");
	if(f)
		fclose(f);
	else
	{
		*filename = '\0';
		return false;
	}
	
	if(! strcmp(filename+l-4, ".tga")) // file is a tga-file
		return true;
	
	// compute name for the tga-file
	int i;
	for(i=0; i<l; i++)
	{
		FileName0[i] = filename[i];
		FileName1[i] = filename[i];
		if(filename[i] == '.')
		{
			FileName1[i+1]=0;
			lstrcat(FileName1, TEXT("tga"));
			lstrcat(FileName1, "\0");
			
			for(i++; i<l; i++)
				FileName0[i] = filename[i];
			FileName0[i]=0;
			break;
		}
	}
	
	// check if tga-file already exists, otherwise, convert
	f=fopen(FileName1,"rb");
	if(f)
	{
		fclose(f);
		strcpy(filename, FileName1);
		return true;
	}
	
	lstrcat(args, TEXT("convert.exe \""));
	lstrcat(args, FileName0);
	lstrcat(args, TEXT("\" \""));
	lstrcat(args, FileName1);
	lstrcat(args, TEXT("\""));
	
	//LPCSTR currentDirectory = szTempPath;		
	STARTUPINFO startupInfo = {sizeof(STARTUPINFO), NULL, NULL, NULL, NULL, NULL, NULL, NULL,
		NULL, NULL, NULL, NULL, NULL, NULL, NULL};
	PROCESS_INFORMATION ProcessInformation; 
	BOOL ok = CreateProcess(NULL, args, NULL, NULL, false,
		CREATE_DEFAULT_ERROR_MODE | CREATE_NO_WINDOW, NULL, NULL, &startupInfo , &ProcessInformation);
	if(!ok)
	{
		*filename = '\0';
		return false;
	}
	
	// loop until tga file is created
	for( i=0; i<100000; i++)
	{
		FILE *f;
		f=fopen(FileName1,"rb");
		if(f)
		{
			fclose(f);
			strcpy(filename, FileName1);
			return true;
		}
		
		MSG msg;
		if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (GetMessage(&msg, NULL, 0, 0))
			{	
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			} else
				exit(0);
			//return msg.wParam;
		} 
	}
	// The tga-file could not be created for some reason. Maybe original texture file is of a very unusual format.
	*filename = '\0';
	return false;
}
